#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

import os
import sys
import traceback
from tornado.web import (RequestHandler, StaticFileHandler as _StaticFileHandler,
                         ErrorHandler as _ErrorHandler, HTTPError
                        )
from tornado.util import raise_exc_info
from tornado.wsgi import HTTPRequest
from torweb.wrappers import cached_property
from torweb.sessions import SessionStore
from torweb.datastructure import MultiDict, ImmutableMultiDict
from code import interact
import logging

logger = logging.getLogger(__name__)

__all__ = ['BaseHandler', 'StaticFileHandler', 'ErrorHandler', 'XMLRPCHandler', 'JSONRPCHandler', 'WSGIRequest']


class BaseHandler(RequestHandler):

    def initialize(self, debug=False, **kwargs):
        self.debug = debug
        logger.info("self.debug:%s" % self.debug)
        self.kwargs = kwargs

    def prepare(self):
        '''
        before request
        '''
        if self.debug:
            self.get_error_html = self.get_debugger_html
        super(BaseHandler, self).prepare()
        #print 'before request'
    #def initialize(self, debug):
    #    # Since we are using the same Handler class for both debug and normal
    #    # modes, we check for debug flag here. Alternatively, define
    #    # get_error_html in a subclass and pass that class to the Application
    #    # on instantiation.
    #    if debug:
    #        self.get_error_html = self.get_debugger_html

    def write_error(self, status_code, **kwargs):
        """Override to implement custom error pages.

        ``write_error`` may call `write`, `render`, `set_header`, etc
        to produce output as usual.

        If this error was caused by an uncaught exception (including
        HTTPError), an ``exc_info`` triple will be available as
        ``kwargs["exc_info"]``.  Note that this exception may not be
        the "current" exception for purposes of methods like
        ``sys.exc_info()`` or ``traceback.format_exc``.

        For historical reasons, if a method ``get_error_html`` exists,
        it will be used instead of the default ``write_error`` implementation.
        ``get_error_html`` returned a string instead of producing output
        normally, and had different semantics for exception handling.
        Users of ``get_error_html`` are encouraged to convert their code
        to override ``write_error`` instead.
        """
        logger.info("write_error")
        #if hasattr(self, 'get_error_html'):
        #    self.finish(self.get_debugger_html(status_code, **kwargs))
        if hasattr(self, 'get_error_html'):
            if 'exc_info' in kwargs:
                exc_info = kwargs.pop('exc_info')
                kwargs['exception'] = exc_info[1]
                try:
                    # Put the traceback into sys.exc_info()
                    raise_exc_info(exc_info)
                except Exception:
                    self.finish(self.get_error_html(status_code, **kwargs))
            else:
                self.finish(self.get_error_html(status_code, **kwargs))
            return

        if self.settings.get("debug") and "exc_info" in kwargs:
            # in debug mode, try to send a traceback
            self.set_header('Content-Type', 'text/plain')
            for line in traceback.format_exception(*kwargs["exc_info"]):
                self.write(line)
            self.finish()
        else:
            self.finish("<html><title>%(code)d: %(message)s</title>"
                        "<body>%(code)d: %(message)s</body></html>" % {
                            "code": status_code,
                            "message": self._reason,
                        })

    def get_error_html(self, status_code, **kwargs):
        return self.write("%s %s"  % (status_code, kwargs))

    def _handle_request_exception(self, e):
        # super(BaseHandler, self)._handle_request_exception(e)
        logger.info('-'*20+"[_handle_request_exception]"+"-"*20)
        logger.info(traceback.format_exc())
        self.write_error(500, exc_info=sys.exc_info(), traceback=traceback.format_exc())
 
    def get_debugger_html(self, status_code, **kwargs):
        if self.debug:
            from torweb.application import DebugApplication
            assert isinstance(self.application, DebugApplication)
            traceback = self.application.get_current_traceback()
            keywords = self.application.get_traceback_renderer_keywords()
            html = traceback.render_full(**keywords).encode('utf-8', 'replace')
            return html.replace(b'WSGI', b'tornado')

    @property
    def args(self):
        names = self.request.arguments.keys()
        arguments = {}
        for name in names:
            arguments[name] = self.get_argument(name)
        return arguments

    @property
    def query_args(self):
        cls = ImmutableMultiDict
        result = []
        keys = []
        querys = self.request.query
        querys = querys.split('&')
        for query_str in querys:
            if not query_str:continue
            if '=' in query_str:
                key, value = query_str.split('=')
            else:
                key = query_str; value = u''
            result.append((key, value.decode('utf-8', 'ignore')))
        return cls(result)

    def on_finish(self):
        '''
        after request
        '''
        pass

    @cached_property
    def session(self):
        '''根据session_sid值来获取session对象，或者初始化一个session对象'''
        session_store = self.application.session_store
        sid = self.cookies.get('session_id')
        #print "sid--> %s" % sid
        if session_store.__class__.__name__ == 'RedisSessionStore':
            if sid is None:
                _sessionsid = self.application.session_store.generate_sid()
                #print "cookie generate_sid--%s" % _sessionsid
            else:
                _sessionsid = sid.value
                #print "cookie sessionsid--%s" % _sessionsid
            from torweb.sessions import RedisSession
            return RedisSession(self.application.session_store, _sessionsid)
        if session_store.__class__.__name__ == 'RedisSessionStoreNew':
            if sid is None:
                _sessionsid = self.application.session_store.generate_sid()
                #print "cookie generate_sid--%s" % _sessionsid
            else:
                _sessionsid = sid.value
                #print "cookie sessionsid--%s" % _sessionsid
            from torweb.sessions import RedisSession
            return RedisSession(self.application.session_store, _sessionsid)
        else:
            #print "session_store -- %s" % session_store
            if sid is None:
                return session_store.new()
            else:
                return session_store.get(sid.value)
            #sessionsid = self.cookies.get("session_id")

    def save_session(self):
        '''返回response之前，向客户端写入cookie
        保存session_sid到cookie
        '''
        try:
            self.set_cookie('session_id', self.session.sid, expires_days=3650)
        except AttributeError:
            pass
        if self.session.should_save:
            self.application.session_store.save(self.session)

    def write_jsonp(self, obj):
        key = self.get_argument("callback", u"jsonpcallback")
        try:
            from simplejson import dumps
        except:
            from json import dumps
        s = dumps(obj)
        self.set_header("Content-Type", "application/json")
        self.write(key+u'(' + s + u')')

class StaticFileHandler(_StaticFileHandler):

    def initialize(self, path, default_filename=None, debug=False, **kwargs):
        super(StaticFileHandler, self).initialize(path, default_filename)
        self.debug = debug
        self.kwargs = kwargs
    static_handler = True

class ErrorHandler(_ErrorHandler): 
    """Generates an error response with status_code for all requests.""" 

    def __init__(self, application, request, status_code): 
        RequestHandler.__init__(self, application, request)
        self.set_status(status_code) 

    def initialize(self, app=None, request=None, status_code=404):
        self.app = app; self.status_code = status_code

    def prepare(self): 
        raise HTTPError(self._status_code)

#    def get_error_html(self, status_code, **kwargs): 
#        from ivtime import base_path
#        return open(os.path.join(base_path,'static', 'error','404.html')).read() 

def private(func):
    # Decorator to make a method, well, private.
    class PrivateMethod(object):
        def __init__(self):
            self.private = True
        __call__ = func
    return PrivateMethod()

class XMLRPCHandler(BaseHandler):

    def get(self):
        pass

    def post(self):
        import xmlrpclib
        try:
            params, method_name = xmlrpclib.loads(self.request.body)
        except:
            # Bad request formatting, bad.
            raise Exception('Deal with how you want.')
        if method_name in dir(RequestHandler):
            # Pre-existing, not an implemented attribute
            raise AttributeError('%s is not implemented.' % method_name)
        try:
            method = getattr(self, method_name)
        except:
            # Attribute doesn't exist
            raise AttributeError('%s is not a valid method.' % method_name)
        if not callable(method):
            # Not callable, so not a method
            raise Exception('Attribute %s is not a method.' % method_name)
        if method_name.startswith('_') or \
                ('private' in dir(method) and method.private is True):
            # No, no. That's private.
            raise Exception('Private function %s called.' % method_name)
        response = method(*params)
        response_xml = xmlrpclib.dumps((response,), methodresponse=True)
        self.set_header("Content-Type", "text/xml")
        self.write(response_xml)


class JSONRPCHandler(BaseHandler):

    def get(self, *args, **kwargs):
        return self.post()

    def post(self, *args, **kwargs):
        """
            JSON-RPC 2.0
            Request Object:
                {u'jsonrpc': u'2.0', u'params': [1, 2], u'id': u'3ebm619c', u'method': u'add'}
            Response Object:
                {"jsonrpc": "2.0", "result": 3, "id": "3ebm619c"}
        """
        import jsonrpclib
        try:
            json_request = jsonrpclib.loads(self.request.body)
            params = json_request.get("params", [])
            method_name = json_request.get("method", "")
        except:
            # Bad request formatting, bad.
            raise Exception('Deal with how you want.')
        if method_name in dir(RequestHandler):
            # Pre-existing, not an implemented attribute
            raise AttributeError('%s is not implemented.' % method_name)
        try:
            method = getattr(self, method_name)
        except:
            # Attribute doesn't exist
            raise AttributeError('%s is not a valid method.' % method_name)
        if not callable(method):
            # Not callable, so not a method
            raise Exception('Attribute %s is not a method.' % method_name)
        if method_name.startswith('_') or \
                ('private' in dir(method) and method.private is True):
            # No, no. That's private.
            raise Exception('Private function %s called.' % method_name)
        response = method(*params)
        response_json = jsonrpclib.dumps(response, methodresponse=True, rpcid=json_request["id"])
        self.set_header("Content-Type", "text/json")
        self.write(response_json)


class WSGIRequest(HTTPRequest):
    def __init__(self, env):
        super(WSGIRequest, self).__init__(env)
        #HTTPRequest.__init__(env)
        self.environ = env




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import soaplib

from lxml import etree

def nillable_value(func):
    def wrapper(cls, value, tns, parent_elt, *args, **kwargs):
        if value is None:
            Null.to_xml(value, tns, parent_elt, *args, **kwargs)
        else:
            func(cls, value, tns, parent_elt, *args, **kwargs)
    return wrapper

def nillable_element(func):
    def wrapper(cls, element):
        if bool(element.get('{%s}nil' % soaplib.ns_xsi)): # or (element.text is None and len(element.getchildren()) == 0):
            return None
        else:
            return func(cls, element)
    return wrapper

def string_to_xml(cls, value, tns, parent_elt, name):
    assert isinstance(value, str) or isinstance(value, unicode), "'value' must " \
                    "be string or unicode. it is instead '%s'" % repr(value)

    element = etree.SubElement(parent_elt, "{%s}%s" % (tns,name))

    element.set('{%s}type' % soaplib.ns_xsi, cls.get_type_name_ns())
    element.text = value

class Base(object):
    __namespace__ = None
    __type_name__ = None

    # There are not the xml schema defaults. The xml schema defaults are
    # considered in ClassSerializer's add_to_schema method. the defaults here
    # are to reflect what people want most.
    class Attributes(object):
        nillable = True
        min_occurs = 0
        max_occurs = 1

    class Empty(object):
        pass

    @staticmethod
    def is_default(cls):
        return True

    @classmethod
    def get_namespace_prefix(cls):
        ns = cls.get_namespace()

        retval = soaplib.get_namespace_prefix(ns)

        return retval

    @classmethod
    def get_namespace(cls):
        return cls.__namespace__

    @staticmethod
    def resolve_namespace(cls, default_ns):
        if cls.__namespace__ in soaplib.const_prefmap and not cls.is_default(cls):
            cls.__namespace__ = default_ns

        if cls.__namespace__ is None:
            cls.__namespace__ = cls.__module__

    @classmethod
    def get_type_name(cls):
        retval = cls.__type_name__
        if retval is None:
            retval = cls.__name__.lower()

        return retval

    @classmethod
    def get_type_name_ns(cls):
        if cls.get_namespace() != None:
            return "%s:%s" % (cls.get_namespace_prefix(), cls.get_type_name())

    @classmethod
    def to_xml(cls, value, tns, parent_elt, name='retval'):
        string_to_xml(cls, value, tns, parent_elt, name)

    @classmethod
    def add_to_schema(cls, schema_entries):
        '''
        Add this type to the wsdl.
        '''
        #Nothing needs to happen when the type is a standard schema element

    @classmethod
    def customize(cls, **kwargs):
        """
        This function duplicates and customizes the class it belongs to. The
        original class remains unchanged. This is an ugly hack. If you know
        better, let us know.
        """

        cls_dict = {}

        for k in cls.__dict__:
            if not (k in ("__dict__", "__weakref__")):
                cls_dict[k] = cls.__dict__[k]

        class Attributes(cls.Attributes):
            pass

        cls_dict['Attributes'] = Attributes

        if not ('_is_clone_of' in cls_dict):
            cls_dict['_is_clone_of'] = cls

        for k,v in kwargs.items():
            setattr(Attributes,k,v)

        cls_dup = type(cls.__name__, cls.__bases__, cls_dict)
        return cls_dup

class Null(Base):
    @classmethod
    def to_xml(cls, value, tns, parent_elt, name='retval'):
        element = etree.SubElement(parent_elt, "{%s}%s" % (tns,name))
        element.set('{%s}nil' % soaplib.ns_xsi, 'true')

    @classmethod
    def from_xml(cls, element):
        return None

class SimpleType(Base):
    __namespace__ = "http://www.w3.org/2001/XMLSchema"
    __base_type__ = None

    class Attributes(Base.Attributes):
        values = set()

    def __new__(cls, **kwargs):
        """
        Overriden so that any attempt to instantiate a primitive will return a
        customized class instead of an instance.

        See serializers.base.Base for more information.
        """

        retval = cls.customize(**kwargs)

        if not retval.is_default(retval):
            retval.__base_type__ = cls
            if retval.__type_name__ is None:
                retval.__type_name__ = kwargs.get("type_name", Base.Empty)

        return retval

    @staticmethod
    def is_default(cls):
        return (cls.Attributes.values == SimpleType.Attributes.values)

    @classmethod
    def get_restriction_tag(cls, schema_entries):
        simple_type = etree.Element('{%s}simpleType' % soaplib.ns_xsd)
        simple_type.set('name', cls.get_type_name())
        schema_entries.add_simple_type(cls, simple_type)

        restriction = etree.SubElement(simple_type,
                                            '{%s}restriction' % soaplib.ns_xsd)
        restriction.set('base', cls.__base_type__.get_type_name_ns())

        for v in cls.Attributes.values:
            enumeration = etree.SubElement(restriction,
                                            '{%s}enumeration' % soaplib.ns_xsd)
            enumeration.set('value', str(v))

        return restriction

    @classmethod
    def add_to_schema(cls, schema_entries):
        if not schema_entries.has_class(cls) and not cls.is_default(cls):
            cls.get_restriction_tag(schema_entries)




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import os
import logging
logger = logging.getLogger(__name__)

import twisted.web.server
import twisted.web.static
from twisted.web.wsgi import WSGIResource
from twisted.internet import reactor

def run_twisted(apps, port):
    """
    takes a list of tuples containing application, url pairs, and a port to
    to listen to.
    """

    static_dir = os.path.abspath(".")
    logging.info("registering static folder %r on /" % static_dir)
    root = twisted.web.static.File(static_dir)

    for app,url in apps:
        resource = WSGIResource(reactor, reactor, app)
        logging.info("registering %r on /%s" % (app, url))
        root.putChild(url, resource)

    site = twisted.web.server.Site(root)

    reactor.listenTCP(port, site)
    logging.info("listening on: 0.0.0.0:%d" % port)

    return reactor.run()



from soaplib.xml import create_xml_element, create_xml_subelement
from soaplib.xml import NamespaceLookup, ElementTree
from soaplib.serializers.primitive import Fault
from soaplib.serializers.binary import Attachment

try:
    #python 2.5
    from email.mime.multipart import MIMEMultipart
    from email.mime.application import MIMEApplication
    from email.encoders import encode_7or8bit
except:
    #python 2.4
    from email.MIMENonMultipart import MIMENonMultipart
    from email.MIMEMultipart import MIMEMultipart
    from email.Encoders import encode_7or8bit

from email import message_from_string


from base64 import b64encode
from StringIO import StringIO

class Message(object):
    def __init__(self,name,params,ns=None,typ=None):
        self.name = name
        self.params = params
        if typ == None:
            typ = name
        self.typ = typ
        self.ns = ns
        
    def to_xml(self,*data):
        if len(self.params):
            if len(data) != len(self.params):
                raise Exception("Parameter number mismatch expected [%s] got [%s]"%(len(self.params),len(data)))
  
        nsmap = NamespaceLookup(self.ns)
        print self.ns
        element = create_xml_element('{%s}%s' % (self.ns,self.name), nsmap)
        
        for i in range(0,len(self.params)):
            name, serializer = self.params[i]
            d = data[i]
            e = serializer.to_xml(d, name, nsmap)
            if type(e) in (list,tuple):
                elist = e
                for e in elist:
                    element.append(e)
            elif e == None:
                pass
            else:
                element.append(e)    
      
        ElementTree.cleanup_namespaces(element)    
        return element
        
    def from_xml(self,element):
        results = []        
        try:
            children = element.getchildren()
        except:
            return []
        
        def findall(name):
            # inner method for finding child node
            nodes = []
            for c in children:
                if c.tag.split('}')[-1] == name:
                    nodes.append(c)
            return nodes
                
        for name, serializer in self.params:
            childnodes = findall(name)
            if len(childnodes) == 0:
                results.append(None)
            else:
                results.append(serializer.from_xml(*childnodes))
        return results
        
    def add_to_schema(self,schemaDict, nsmap):
        complexType = create_xml_element(nsmap.get('xs') + 'complexType', nsmap)
        complexType.set('name',self.typ)
        
        sequence = create_xml_subelement(complexType, nsmap.get('xs') + 'sequence')
        if self.params:
            for name,serializer in self.params:
                e = create_xml_subelement(sequence, nsmap.get('xs') + 'element')
                e.set('name',name)
                e.set('type',
                    "%s:%s" % (serializer.get_namespace_id(), serializer.get_datatype()))
                
        element = create_xml_element(nsmap.get('xs') + 'element', nsmap)
        element.set('name',self.typ)
        element.set('type','%s:%s' % ('tns',self.typ))
        
        schemaDict[self.typ] = complexType
        schemaDict[self.typ+'Element'] = element
        
class MethodDescriptor:
    '''
    This class represents the method signature of a soap method, and is returned
    by the soapdocument, or soapmethod decorators.  
    '''

    def __init__(self, name, soapAction, inMessage, outMessage, doc, isCallback=False, isAsync=False, mtom=False):
        self.inMessage = inMessage
        self.outMessage = outMessage
        self.soapAction = soapAction
        self.name = name
        self.isCallback = isCallback
        self.isAsync = isAsync
        self.doc = doc
        self.mtom = mtom

def from_soap(xml_string):
    '''
    Parses the xml string into the header and payload
    '''
    root, xmlids = ElementTree.XMLID(xml_string)    
    if xmlids:
        resolve_hrefs(root,xmlids)
    body = None
    header = None

    # find the body and header elements
    for e in root.getchildren():
        name = e.tag.split('}')[-1].lower()
        if name == 'body':
            body = e
        elif name == 'header':
            header = e
    payload = None
    if len(body.getchildren()):
        payload = body.getchildren()[0]

    return payload, header

def resolve_hrefs(element,xmlids):
    for e in element:
        if e.get('id'): 
            continue # don't need to resolve this element    
        elif e.get('href'):
            resolved_element = xmlids[e.get('href').replace('#','')]
            if resolved_element is None: continue
            resolve_hrefs(resolved_element,xmlids)

            [e.set(k,v) for k,v in resolved_element.items()]              # copies the attributes
            [e.append(child) for child in resolved_element.getchildren()] # copies the children

        else:
            resolve_hrefs(e,xmlids)
    return element


def make_soap_envelope(message, tns='', header_elements=None):
    '''
    This method takes the results from a soap method call, and wraps them
    in the appropriate soap envelope with any specified headers

    @param the message of the soap envelope, either an element or list of elements
    @param any header elements to be included in the soap response
    @returns the envelope element
    '''
    nsmap = NamespaceLookup(tns)
    
    envelope = create_xml_element(nsmap.get('SOAP-ENV') + 'Envelope', nsmap)

    if header_elements:
        headerElement = create_xml_subelement(envelope, nsmap.get('SOAP-ENV') + 'Header')
        
        for h in header_elements:
            headerElement.append(h)
   
    body = create_xml_subelement(envelope, nsmap.get('SOAP-ENV') + 'Body')

    if type(message) == list:
        for m in message:
            body.append(m)
    elif message != None:
        body.append(message)

    return envelope

def join_attachment(id, envelope, payload, prefix=True):
    '''
    Helper function for swa_to_soap.
    
    Places the data from an attachment back into a SOAP message, replacing
    its xop:Include element or href.
    
    @param  id          content-id or content-location of attachment
    @param  prefix      Set this to true if id is content-id or false if 
                        it is content-location.  It prefixes a "cid:" to
                        the href value.
    @param  envelope    soap envelope string to be operated on
    @param  payload     attachment data
    @return             tuple of length 2 with the new message and the
                        number of replacements made
    '''
    
    # grab the XML element of the message in the SOAP body
    soapmsg = StringIO(envelope)
    soaptree = ElementTree.parse(soapmsg)
    soapns = soaptree.getroot().tag.split('}')[0].strip('{')
    soapbody = soaptree.getroot().find("{%s}Body" % soapns)
    message = None
    for child in list(soapbody):
        if child.tag != "%sFault" % (soapns,):
            message = child
            break
    
    numreplaces = 0
    idprefix = ''
    if prefix == True: idprefix = "cid:"
    id = "%s%s" % (idprefix, id, )
    
    # Make replacement.
    for param in message:
        # Look for Include subelement.
        for sub in param:
            if sub.tag.split('}')[-1] == 'Include' and \
               sub.attrib.get('href') == id:
                param.remove(sub)
                param.text = payload
                numreplaces += 1
        if numreplaces < 1 and param.attrib.get('href') == id:
            del(param.attrib['href'])
            param.text = payload
            numreplaces += 1
            
    soapmsg.close()
    soapmsg = StringIO()
    soaptree.write(soapmsg)
    joinedmsg = soapmsg.getvalue()
    soapmsg.close()
    
    return (joinedmsg, numreplaces)

def collapse_swa(content_type, envelope):
    '''
    Translates an SwA multipart/related message into an application/soap+xml message.
    
    References:
    SwA     http://www.w3.org/TR/SOAP-attachments
    XOP     http://www.w3.org/TR/xop10/
    MTOM    http://www.w3.org/TR/soap12-mtom/
            http://www.w3.org/Submission/soap11mtom10/
            
    @param  content_type value of the Content-Type header field
    @param  envelope     body of the HTTP message, a soap envelope
    @return              appication/soap+xml version of the given HTTP body
    '''
    # convert multipart messages back to pure SOAP
    mime_type = content_type.lower().split(';')
    if 'multipart/related' not in mime_type:
        return envelope
        
    # parse the body into an email.Message object
    msgString = "MIME-Version: 1.0\r\n" \
                "Content-Type: %s\r\n" % (
                content_type, )             
    msgString += "\r\n" + envelope                
    msg = message_from_string(msgString) # our message
    
    soapmsg = None
    root = msg.get_param('start')

    _unmentioned_attachs = []
    # walk through sections, reconstructing pure SOAP
    for part in msg.walk():
        # skip the multipart container section
        if part.get_content_maintype() == 'multipart':
            continue
        
        # detect main soap section
        if (part.get('Content-ID') and part.get('Content-ID') == root) or \
           (root == None and part == msg.get_payload()[0]):
            soapmsg = part.get_payload()
            continue
           
        # binary packages
        cte = part.get("Content-Transfer-Encoding")
        payload = None
        if cte != 'base64':
            payload = b64encode(part.get_payload())
        else:
            payload = part.get_payload()
        cid = part.get("Content-ID").strip("<>")
        cloc = part.get("Content-Location")
        numreplaces = None
        
        # Check for Content-ID and make replacement
        if cid:
            soapmsg, numreplaces = join_attachment(cid, soapmsg, payload)
            if numreplaces == 0:
                _unmentioned_attachs.append(part)
            
        # Check for Content-Location and make replacement
        if cloc and not cid and not numreplaces:
            soapmsg, numreplaces = join_attachment(cloc,soapmsg,payload,False)
                
    return soapmsg, _unmentioned_attachs

def apply_mtom(headers, envelope, params, paramvals):
    '''
    Apply MTOM to a SOAP envelope, separating attachments into a
    MIME multipart message.
    
    References:
    XOP     http://www.w3.org/TR/xop10/
    MTOM    http://www.w3.org/TR/soap12-mtom/
            http://www.w3.org/Submission/soap11mtom10/
    
    @param headers   Headers dictionary of the SOAP message that would 
                     originally be sent.
    @param envelope  SOAP envelope string that would have originally been sent.
    @param params    params attribute from the Message object used for the SOAP
    @param paramvals values of the params, passed to Message.to_xml
    @return          tuple of length 2 with dictionary of headers and 
                     string of body that can be sent with HTTPConnection
    '''
    
    # grab the XML element of the message in the SOAP body
    soapmsg = StringIO(envelope)
    soaptree = ElementTree.parse(soapmsg)
    soapns = soaptree.getroot().tag.split('}')[0].strip('{')
    soapbody = soaptree.getroot().find("{%s}Body" % soapns)
    message = None
    for child in list(soapbody):
        if child.tag != "%sFault" % (soapns,):
            message = child
            break
    
    # Get additional parameters from original Content-Type
    ctarray = []
    for n, v in headers.items():
        if n.lower() == 'content-type':
            ctarray = v.split(';')
            break
    roottype = ctarray[0].strip()
    rootparams = {}
    for ctparam in ctarray[1:]:
        n, v = ctparam.strip().split('=')
        rootparams[n] = v.strip("\"'")
    
    # Set up initial MIME parts
    mtompkg = MIMEMultipart('related',boundary='?//<><>soaplib_MIME_boundary<>')
    rootpkg = None
    try:
        rootpkg = MIMEApplication(envelope,'xop+xml', encode_7or8bit)
    except NameError:
        rootpkg = MIMENonMultipart("application", "xop+xml")
        rootpkg.set_payload(envelope)
        encode_7or8bit(rootpkg)
    
    # Set up multipart headers.
    del(mtompkg['mime-version'])
    mtompkg.set_param('start-info',roottype)
    mtompkg.set_param('start','<soaplibEnvelope>')
    if headers.has_key('SOAPAction'):
        mtompkg.add_header('SOAPAction', headers.get('SOAPAction'))
    
    # Set up root SOAP part headers
    del(rootpkg['mime-version'])
    rootpkg.add_header('Content-ID','<soaplibEnvelope>')
    for n, v in rootparams.items():
        rootpkg.set_param(n,v)
    rootpkg.set_param('type',roottype)
    
    mtompkg.attach(rootpkg)
    
    # Extract attachments from SOAP envelope.
    for i in range(len(params)):
        name, typ = params[i]
        if typ == Attachment:
            id = "soaplibAttachment_%s" % (len(mtompkg.get_payload()),)
            param = message[i]
            param.text = ""
            incl = create_xml_subelement(
                       param, 
                       "{http://www.w3.org/2004/08/xop/include}Include"
                   )
            incl.attrib["href"] = "cid:%s" % id
            if paramvals[i].fileName and not paramvals[i].data:
                paramvals[i].load_from_file()
            data = paramvals[i].data
            attachment = None
            try:
                attachment = MIMEApplication(data, _encoder=encode_7or8bit)
            except NameError:
                attachment = MIMENonMultipart("application", "octet-stream")
                attachment.set_payload(data)
                encode_7or8bit(attachment)
            del(attachment['mime-version'])
            attachment.add_header('Content-ID','<%s>'%(id,))
            mtompkg.attach(attachment)
    
    # Update SOAP envelope.
    soapmsg.close()
    soapmsg = StringIO()
    soaptree.write(soapmsg)
    rootpkg.set_payload(soapmsg.getvalue())
    soapmsg.close()
    
    # extract body string from MIMEMultipart message
    bound = '--%s' % (mtompkg.get_boundary(),)
    marray = mtompkg.as_string().split(bound)
    mtombody = bound
    mtombody += bound.join(marray[1:])
    
    # set Content-Length
    mtompkg.add_header("Content-Length", str(len(mtombody)))
    
    # extract dictionary of headers from MIMEMultipart message
    mtomheaders = {}
    for name, value in mtompkg.items():
        mtomheaders[name] = value
    
    if len(mtompkg.get_payload()) <= 1:
        return (headers, envelope)
    
    return (mtomheaders, mtombody)

def make_soap_fault(faultString, faultCode = 'Server', detail = None, 
    header_elements = None):
    '''
    This method populates a soap fault message with the provided 
    fault string and details.  
    @param faultString the short description of the error
    @param detail the details of the exception, such as a stack trace
    @param faultCode defaults to 'Server', but can be overridden
    @param header_elements A list of XML elements to add to the fault header.
    @returns the element corresponding to the fault message
    '''
    
    nsmap = NamespaceLookup()
    envelope = create_xml_element(nsmap.get('SOAP-ENV') + 'Envelope', nsmap)
    
    if header_elements:
        header = create_xml_subelement(
            envelope, nsmap.get('SOAP-ENV') + 'Header')
        for element in header_elements:
            header.append(element)
    
    body = create_xml_subelement(envelope, nsmap.get('SOAP-ENV') + 'Body')

    f = Fault(faultCode,faultString,detail)
    body.append(Fault.to_xml(f, nsmap.get('SOAP-ENV') + "Fault", nsmap))

    return envelope



from soaplib.xml import ns, create_xml_element, create_xml_subelement
from soaplib.etimport import ElementTree
import datetime
import re
import cStringIO 

import pytz
from pytz import FixedOffset

#######################################################
# Utility Functions
#######################################################

string_encoding = 'utf-8'

_datetime_pattern = r'(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})[T ](?P<hr>\d{2}):(?P<min>\d{2}):(?P<sec>\d{2})(?P<fractional_sec>\.\d+)?'
_local_re = re.compile(_datetime_pattern)
_utc_re = re.compile(_datetime_pattern + 'Z')
_offset_re = re.compile(_datetime_pattern + r'(?P<tz_hr>[+-]\d{2}):(?P<tz_min>\d{2})')

def _is_null_element(element):
    for k in element.keys():
        if k.endswith('null'):
            return True
    return False

def _element_to_datetime(element):
    # expect ISO formatted dates
    # 
    text = element.text
    if not text:
        return None
    
    def parse_date(date_match, tz=None):
        fields = date_match.groupdict(0)
        year, month, day, hr, min, sec = [ int(fields[x]) for x in 
           ("year", "month", "day", "hr", "min", "sec")]
        # use of decimal module here (rather than float) might be better
        # here, if willing to require python 2.4 or higher
        microsec = int(float(fields.get("fractional_sec", 0)) * 10**6)
        return datetime.datetime(year, month, day, hr, min, sec, microsec, tz)
    
    match = _utc_re.match(text)
    if match:
        return parse_date(match, tz=pytz.utc)
    match = _offset_re.match(text)
    if match:
        tz_hr, tz_min = [int(match.group(x)) for x in "tz_hr", "tz_min"]
        return parse_date(match, tz=FixedOffset(tz_hr*60 + tz_min, {}))
    match = _local_re.match(text)
    if match:
        return parse_date(match)
    raise Exception("DateTime [%s] not in known format"%text)

def _element_to_string(element):
    text = element.text
    if text:
        return text.decode(string_encoding)
    else:
        return None

def _element_to_integer(element):
    i = element.text
    if not i:
        return None
    try: 
        return int(str(i))
    except: 
        try: return long(i)
        except: return None

def _element_to_float(element):
    f = element.text
    if f is None:
        return None
    return float(f)

def _element_to_unicode(element):
    u = element.text
    if not u:
        return None
    try:
       u = str(u)
       return u.encode(string_encoding)
    except:
       return u

def _unicode_to_xml(value, name, cls, nsmap):
    retval = create_xml_element(name, nsmap)
    if value == None:
        return Null.to_xml(value,name,nsmap)
    if type(value) == unicode:
        retval.text = value
    else: 
        retval.text = unicode(value,string_encoding)
    retval.set(
        nsmap.get('xsi') + 'type', 
        "%s:%s" % (cls.get_namespace_id(), cls.get_datatype()))
    return retval

def _generic_to_xml(value, name, cls, nsmap):
    retval = create_xml_element(name, nsmap)
    if value:
        retval.text = value
    retval.set(
        nsmap.get('xsi') + 'type',
        "%s:%s" % (cls.get_namespace_id(), cls.get_datatype()))
    return retval
    
def _get_datatype(cls, typename, nsmap):
    if nsmap is not None:
        return nsmap.get(cls.get_namespace_id()) + typename
    return typename

class Any:

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        if type(value) == str:
            value = ElementTree.fromstring(value)
        e = create_xml_element(name, nsmap)
        e.append(value) 
        return e 
        
    @classmethod
    def from_xml(cls,element):
        children = element.getchildren()
        if children:
            return element.getchildren()[0]
        return None

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'anyType', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass

class String:

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        e = _unicode_to_xml(value, name, cls, nsmap)
        return e
        
    @classmethod
    def from_xml(cls,element):
        return _element_to_unicode(element)

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'string', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass

class Fault(Exception):

    def __init__(self, faultcode = 'Server', faultstring = None, detail = None, name = 'ExceptionFault'):
        self.faultcode = faultcode
        self.faultstring = faultstring
        self.detail = detail
        self.name = name

    @classmethod
    def to_xml(cls, value, name, nsmap=ns):
        fault = create_xml_element(name, nsmap)
        create_xml_subelement(fault, 'faultcode').text = value.faultcode
        create_xml_subelement(fault, 'faultstring').text = value.faultstring
        detail = create_xml_subelement(fault, 'detail').text = value.detail
        return fault


    @classmethod
    def from_xml(cls, element):
        code = _element_to_string(element.find('faultcode'))
        string = _element_to_string(element.find('faultstring'))
        detail_element = element.find('detail')
        if detail_element is not None:
            if len(detail_element.getchildren()):
                detail = ElementTree.tostring(detail_element)
            else:
                detail = _element_to_string(element.find('detail'))
        else:
            detail = ''
        return Fault(faultcode = code, faultstring = string, detail = detail)

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'ExceptionFaultType', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'tns'

    @classmethod
    def add_to_schema(cls,schema_dict,nsmap):   
        complexTypeNode = create_xml_element('complexType', nsmap)
        complexTypeNode.set('name', cls.get_datatype())        
        sequenceNode = create_xml_subelement(complexTypeNode, 'sequence')
        faultTypeElem = create_xml_subelement(sequenceNode,'element')
        faultTypeElem.set('name','detail')
        faultTypeElem.set(nsmap.get('xsi') + 'type', 'xs:string')
        faultTypeElem = create_xml_subelement(sequenceNode,'element')
        faultTypeElem.set('name','message')
        faultTypeElem.set(nsmap.get('xsi') + 'type', 'xs:string')
    
        schema_dict[cls.get_datatype()] = complexTypeNode
        
        typeElementItem = create_xml_element('element', nsmap)
        typeElementItem.set('name', 'ExceptionFaultType')
        typeElementItem.set(nsmap.get('xsi') + 'type', cls.get_datatype(nsmap))
        schema_dict['%sElement'%(cls.get_datatype(nsmap))] = typeElementItem
        
    def __str__(self):
        io = cStringIO.StringIO()
        io.write("*"*80)
        io.write("\r\n")
        io.write(" Recieved soap fault \r\n")
        io.write(" FaultCode            %s \r\n"%self.faultcode)
        io.write(" FaultString          %s \r\n"%self.faultstring)
        io.write(" FaultDetail          \r\n")
        if self.detail is not None:
            io.write(self.detail)
        return io.getvalue()

class Integer:

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        e = _generic_to_xml(str(value), name, cls, nsmap)
        return e
    
    @classmethod
    def from_xml(cls,element):
        return _element_to_integer(element)

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'int', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass


class Double:

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        e = _generic_to_xml(str(value), name, cls, nsmap)
        return e

    @classmethod
    def from_xml(cls,element):
        return _element_to_integer(element)

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'double', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass


class DateTime:

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        if type(value) == datetime.datetime:
            value = value.isoformat('T')
        e = _generic_to_xml(value, name, cls, nsmap)    
        return e
    
    @classmethod
    def from_xml(cls,element):
        return _element_to_datetime(element)            

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'dateTime', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass

class Float:

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        e = _generic_to_xml(str(value), name, cls, nsmap)
        return e
    
    @classmethod
    def from_xml(cls,element):
        return _element_to_float(element)

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'float', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass

class Null:

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        element = create_xml_element(name, nsmap)
        element.set(cls.get_datatype(nsmap),'1')
        return element
    
    @classmethod
    def from_xml(cls,element):
        return None

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'null', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass

class Boolean:
    
    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        # applied patch from Julius Volz
        #e = _generic_to_xml(str(value).lower(),name,cls.get_datatype(nsmap))    
        if value == None:
            return Null.to_xml('', name, nsmap)
        else:
            e = _generic_to_xml(str(bool(value)).lower(), name, cls, nsmap)
        return e
    
    @classmethod
    def from_xml(cls,element):
        s = _element_to_string(element)
        if s == None: 
            return None
        if s and s.lower()[0] == 't':
            return True
        return False

    @classmethod
    def get_datatype(cls, nsmap=None):
        return _get_datatype(cls, 'boolean', nsmap)

    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        pass
    
class Array:
    
    def __init__(self,serializer,type_name=None,namespace_id='tns'):
        self.serializer = serializer
        self.namespace_id = namespace_id
        if not type_name:
            self.type_name = '%sArray'%self.serializer.get_datatype()
        else:
            self.type_name = type_name

    def to_xml(self,values,name='retval',nsmap=ns):
        res = create_xml_element(name, nsmap)
        typ = self.get_datatype(nsmap)
        if values == None:
            values = []
        res.set('type', 
            "%s:%s" % (self.get_namespace_id(), self.get_datatype()))
        for value in values:
            serializer = self.serializer
            if value == None:
                serializer = Null
            res.append(
                serializer.to_xml(value, serializer.get_datatype(), nsmap))
        return res    

    def from_xml(self,element):
        results = []
        for child in element.getchildren():
            results.append(self.serializer.from_xml(child))
        return results

    def get_datatype(self, nsmap=None):
        return _get_datatype(self, self.type_name, nsmap)

    def get_namespace_id(self):
        return self.namespace_id

    def add_to_schema(self,schema_dict,nsmap):
        typ = self.get_datatype()
        
        self.serializer.add_to_schema(schema_dict, nsmap)

        if not schema_dict.has_key(typ):

            complexTypeNode = create_xml_element(
                nsmap.get('xs') + 'complexType', nsmap)
            complexTypeNode.set('name',self.get_datatype())

            sequenceNode = create_xml_subelement(
                complexTypeNode, nsmap.get('xs') + 'sequence')
            elementNode = create_xml_subelement(
                sequenceNode, nsmap.get('xs') + 'element')
            elementNode.set('minOccurs','0')
            elementNode.set('maxOccurs','unbounded')
            print self.serializer.get_namespace_id()
            elementNode.set('type',
                "%s:%s" % (self.serializer.get_namespace_id(), self.serializer.get_datatype()))
            elementNode.set('name',self.serializer.get_datatype())

            typeElement = create_xml_element(
                nsmap.get('xs') + 'element', nsmap)
            typeElement.set('name',typ)
            typeElement.set('type',
                "%s:%s" % (self.namespace_id, self.get_datatype()))
            
            schema_dict['%sElement'%(self.get_datatype(nsmap))] = typeElement
            schema_dict[self.get_datatype(nsmap)] = complexTypeNode

class Repeating(object):

    def __init__(self,serializer,type_name=None,namespace_id='tns'):
        self.serializer = serializer
        self.namespace_id = namespace_id
        
    def to_xml(self,values,name='retval',nsmap=ns):
        if values == None:
            values = []
        res = []
        for value in values:
            serializer = self.serializer
            if value == None:
                serializer = Null
            res.append(
                serializer.to_xml(value,name,nsmap)
            )
        return res  
        
    def get_namespace_id(self):
        return self.namespace_id  

    def from_xml(self,*elements):
        results = []
        for child in elements:
            results.append(self.serializer.from_xml(child))
        return results    
        
    def add_to_schema(self,schema_dict,nsmap):
        raise Exception("The Repeating serializer is experimental and not supported for wsdl generation")



import inspect
from soaplib.xml import ns, create_xml_element, create_xml_subelement, qualify

from primitive import Null

class ClassSerializerMeta(type):
    '''
    This is the metaclass that populates ClassSerializer instances with
    the appropriate datatypes for (de)serialization
    '''

    def __init__(cls, clsname, bases, dictionary):
        '''
        This initializes the class, and sets all the appropriate
        types onto the class for serialization.  This implementation
        assumes that all attributes assigned to this class  are internal 
        serializers for this class
        '''
        if not hasattr(cls,'types'):
            return
        types = cls.types
        members = dict(inspect.getmembers(types))
        cls.soap_members = {}
        cls.namespace = None
        
        for k,v in members.items():
            if k == '_namespace_':
                cls.namespace=v
                
            elif not k.startswith('__'):
                cls.soap_members[k] = v
        
        # COM bridge attributes that are otherwise harmless
        cls._public_methods_ = []
        cls._public_attrs_ = cls.soap_members.keys()

class ClassSerializer(object):
    __metaclass__ = ClassSerializerMeta

    def __init__(self):
        cls = self.__class__
        for k,v in cls.soap_members.items():
            setattr(self,k,None)

    @classmethod
    def to_xml(cls,value,name='retval', nsmap=ns):
#        element = create_xml_element(
#            nsmap.get(cls.get_namespace_id()) + name, nsmap)
        element = create_xml_element(name, nsmap)
        element.set('type', '%s:%s'%(cls.get_namespace_id(), cls.get_datatype()))
                                     
        for k,v in cls.soap_members.items():
            member_value = getattr(value,k,None)    

            subvalue = getattr(value,k,None)
            if subvalue is None:
                v = Null
                
            subelements = v.to_xml(subvalue,name=k,nsmap=nsmap)
            if type(subelements) != list:
                subelements = [subelements]
            for s in subelements:
                element.append(s)
        return element

    @classmethod
    def from_xml(cls, element):
        obj = cls()
        children = element.getchildren()
        d = {}
        for c in children:
            name = c.tag.split('}')[-1]
            if not d.has_key(name):
                d[name] = []
            d[name].append(c)
            
        for tag,v in d.items():
            member = cls.soap_members.get(tag)
            
            value = member.from_xml(*v)
            setattr(obj,tag,value)
        return obj

    @classmethod
    def get_datatype(cls,nsmap=None):
        if nsmap is not None:
            return nsmap.get(cls.get_namespace_id()) + cls.__name__
        return cls.__name__
        
    @classmethod
    def get_namespace_id(cls):
        return 'tns'

    @classmethod
    def add_to_schema(cls, schemaDict, nsmap):
        
        if not schemaDict.has_key(cls.get_datatype(nsmap)):
            for k,v in cls.soap_members.items():
                v.add_to_schema(schemaDict, nsmap)

            schema_node = create_xml_element(
                nsmap.get("xs") + "complexType", nsmap)
            schema_node.set('name',cls.__name__)

            sequence_node = create_xml_subelement(
                schema_node, nsmap.get('xs') + 'sequence')
            for k,v in cls.soap_members.items():
                member_node = create_xml_subelement(
                    sequence_node, nsmap.get('xs') + 'element')
                member_node.set('name',k)
                member_node.set('minOccurs','0')
                member_node.set('type',
                    "%s:%s" % (v.get_namespace_id(), v.get_datatype()))

            typeElement = create_xml_element(
                nsmap.get('xs') + 'element', nsmap)
            typeElement.set('name',cls.__name__)
            typeElement.set('type',
                "%s:%s" % (cls.get_namespace_id(),cls.__name__))
            schemaDict[cls.get_datatype(nsmap)+'Complex'] = schema_node
            schemaDict[cls.get_datatype(nsmap)] = typeElement



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

from torweb.wrappers import _missing

def is_immutable(self):
    raise TypeError('%r objects are immutable' % self.__class__.__name__)

class TypeConversionDict(dict):
    """Works like a regular dict but the :meth:`get` method can perform
    type conversions.  :class:`MultiDict` and :class:`CombinedMultiDict`
    are subclasses of this class and provide the same feature.

    .. versionadded:: 0.5
    """

    def get(self, key, default=None, type=None):
        """Return the default value if the requested data doesn't exist.
        If `type` is provided and is a callable it should convert the value,
        return it or raise a :exc:`ValueError` if that is not possible.  In
        this case the function will return the default as if the value was not
        found:

        >>> d = TypeConversionDict(foo='42', bar='blub')
        >>> d.get('foo', type=int)
        42
        >>> d.get('bar', -1, type=int)
        -1

        :param key: The key to be looked up.
        :param default: The default value to be returned if the key can't
                        be looked up.  If not further specified `None` is
                        returned.
        :param type: A callable that is used to cast the value in the
                     :class:`MultiDict`.  If a :exc:`ValueError` is raised
                     by this callable the default value is returned.
        """
        try:
            rv = self[key]
            if type is not None:
                rv = type(rv)
        except (KeyError, ValueError):
            rv = default
        return rv

class MultiDict(TypeConversionDict):
    """A :class:`MultiDict` is a dictionary subclass customized to deal with
    multiple values for the same key which is for example used by the parsing
    functions in the wrappers.  This is necessary because some HTML form
    elements pass multiple values for the same key.

    :class:`MultiDict` implements all standard dictionary methods.
    Internally, it saves all values for a key as a list, but the standard dict
    access methods will only return the first value for a key. If you want to
    gain access to the other values, too, you have to use the `list` methods as
    explained below.

    Basic Usage:

    >>> d = MultiDict([('a', 'b'), ('a', 'c')])
    >>> d
    MultiDict([('a', 'b'), ('a', 'c')])
    >>> d['a']
    'b'
    >>> d.getlist('a')
    ['b', 'c']
    >>> 'a' in d
    True

    It behaves like a normal dict thus all dict functions will only return the
    first value when multiple values for one key are found.

    From Werkzeug 0.3 onwards, the `KeyError` raised by this class is also a
    subclass of the :exc:`~exceptions.BadRequest` HTTP exception and will
    render a page for a ``400 BAD REQUEST`` if caught in a catch-all for HTTP
    exceptions.

    A :class:`MultiDict` can be constructed from an iterable of
    ``(key, value)`` tuples, a dict, a :class:`MultiDict` or from Werkzeug 0.2
    onwards some keyword parameters.

    :param mapping: the initial value for the :class:`MultiDict`.  Either a
                    regular dict, an iterable of ``(key, value)`` tuples
                    or `None`.
    """

    # the key error this class raises.  Because of circular dependencies
    # with the http exception module this class is created at the end of
    # this module.
    KeyError = None

    def __init__(self, mapping=None):
        if isinstance(mapping, MultiDict):
            dict.__init__(self, ((k, l[:]) for k, l in mapping.iterlists()))
        elif isinstance(mapping, dict):
            tmp = {}
            for key, value in mapping.iteritems():
                if isinstance(value, (tuple, list)):
                    value = list(value)
                else:
                    value = [value]
                tmp[key] = value
            dict.__init__(self, tmp)
        else:
            tmp = {}
            for key, value in mapping or ():
                tmp.setdefault(key, []).append(value)
            dict.__init__(self, tmp)

    def __getstate__(self):
        return dict(self.lists())

    def __setstate__(self, value):
        dict.clear(self)
        dict.update(self, value)

    def __iter__(self):
        return self.iterkeys()

    def __getitem__(self, key):
        """Return the first data value for this key;
        raises KeyError if not found.

        :param key: The key to be looked up.
        :raise KeyError: if the key does not exist.
        """
        if key in self:
            return dict.__getitem__(self, key)[0]
        raise KeyError

    def __setitem__(self, key, value):
        """Like :meth:`add` but removes an existing key first.

        :param key: the key for the value.
        :param value: the value to set.
        """
        dict.__setitem__(self, key, [value])

    def add(self, key, value):
        """Adds a new value for the key.

        .. versionadded:: 0.6

        :param key: the key for the value.
        :param value: the value to add.
        """
        dict.setdefault(self, key, []).append(value)

    def getlist(self, key, type=None):
        """Return the list of items for a given key. If that key is not in the
        `MultiDict`, the return value will be an empty list.  Just as `get`
        `getlist` accepts a `type` parameter.  All items will be converted
        with the callable defined there.

        :param key: The key to be looked up.
        :param type: A callable that is used to cast the value in the
                     :class:`MultiDict`.  If a :exc:`ValueError` is raised
                     by this callable the value will be removed from the list.
        :return: a :class:`list` of all the values for the key.
        """
        try:
            rv = dict.__getitem__(self, key)
        except KeyError:
            return []
        if type is None:
            return list(rv)
        result = []
        for item in rv:
            try:
                result.append(type(item))
            except ValueError:
                pass
        return result

    def setlist(self, key, new_list):
        """Remove the old values for a key and add new ones.  Note that the list
        you pass the values in will be shallow-copied before it is inserted in
        the dictionary.

        >>> d = MultiDict()
        >>> d.setlist('foo', ['1', '2'])
        >>> d['foo']
        '1'
        >>> d.getlist('foo')
        ['1', '2']

        :param key: The key for which the values are set.
        :param new_list: An iterable with the new values for the key.  Old values
                         are removed first.
        """
        dict.__setitem__(self, key, list(new_list))

    def setdefault(self, key, default=None):
        """Returns the value for the key if it is in the dict, otherwise it
        returns `default` and sets that value for `key`.

        :param key: The key to be looked up.
        :param default: The default value to be returned if the key is not
                        in the dict.  If not further specified it's `None`.
        """
        if key not in self:
            self[key] = default
        else:
            default = self[key]
        return default

    def setlistdefault(self, key, default_list=None):
        """Like `setdefault` but sets multiple values.  The list returned
        is not a copy, but the list that is actually used internally.  This
        means that you can put new values into the dict by appending items
        to the list:

        >>> d = MultiDict({"foo": 1})
        >>> d.setlistdefault("foo").extend([2, 3])
        >>> d.getlist("foo")
        [1, 2, 3]

        :param key: The key to be looked up.
        :param default: An iterable of default values.  It is either copied
                        (in case it was a list) or converted into a list
                        before returned.
        :return: a :class:`list`
        """
        if key not in self:
            default_list = list(default_list or ())
            dict.__setitem__(self, key, default_list)
        else:
            default_list = dict.__getitem__(self, key)
        return default_list

    def items(self, multi=False):
        """Return a list of ``(key, value)`` pairs.

        :param multi: If set to `True` the list returned will have a
                      pair for each value of each key.  Otherwise it
                      will only contain pairs for the first value of
                      each key.

        :return: a :class:`list`
        """
        return list(self.iteritems(multi))

    def lists(self):
        """Return a list of ``(key, value)`` pairs, where values is the list of
        all values associated with the key.

        :return: a :class:`list`
        """
        return list(self.iterlists())

    def values(self):
        """Returns a list of the first value on every key's value list.

        :return: a :class:`list`.
        """
        return [self[key] for key in self.iterkeys()]

    def listvalues(self):
        """Return a list of all values associated with a key.  Zipping
        :meth:`keys` and this is the same as calling :meth:`lists`:

        >>> d = MultiDict({"foo": [1, 2, 3]})
        >>> zip(d.keys(), d.listvalues()) == d.lists()
        True

        :return: a :class:`list`
        """
        return list(self.iterlistvalues())

    def iteritems(self, multi=False):
        """Like :meth:`items` but returns an iterator."""
        for key, values in dict.iteritems(self):
            if multi:
                for value in values:
                    yield key, value
            else:
                yield key, values[0]

    def iterlists(self):
        """Return a list of all values associated with a key.

        :return: a class:`list`
        """
        for key, values in dict.iteritems(self):
            yield key, list(values)

    def itervalues(self):
        """Like :meth:`values` but returns an iterator."""
        for values in dict.itervalues(self):
            yield values[0]

    def iterlistvalues(self):
        """like :meth:`listvalues` but returns an iterator."""
        for values in dict.itervalues(self):
            yield list(values)

    def copy(self):
        """Return a shallow copy of this object."""
        return self.__class__(self)

    def to_dict(self, flat=True):
        """Return the contents as regular dict.  If `flat` is `True` the
        returned dict will only have the first item present, if `flat` is
        `False` all values will be returned as lists.

        :param flat: If set to `False` the dict returned will have lists
                     with all the values in it.  Otherwise it will only
                     contain the first value for each key.
        :return: a :class:`dict`
        """
        if flat:
            return dict(self.iteritems())
        return dict(self.lists())

    def update(self, other_dict):
        """update() extends rather than replaces existing key lists."""
        for key, value in iter_multi_items(other_dict):
            MultiDict.add(self, key, value)

    def pop(self, key, default=_missing):
        """Pop the first item for a list on the dict.  Afterwards the
        key is removed from the dict, so additional values are discarded:

        >>> d = MultiDict({"foo": [1, 2, 3]})
        >>> d.pop("foo")
        1
        >>> "foo" in d
        False

        :param key: the key to pop.
        :param default: if provided the value to return if the key was
                        not in the dictionary.
        """
        try:
            return dict.pop(self, key)[0]
        except KeyError, e:
            if default is not _missing:
                return default
            raise self.KeyError(str(e))

    def popitem(self):
        """Pop an item from the dict."""
        try:
            item = dict.popitem(self)
            return (item[0], item[1][0])
        except KeyError, e:
            raise self.KeyError(str(e))

    def poplist(self, key):
        """Pop the list for a key from the dict.  If the key is not in the dict
        an empty list is returned.

        .. versionchanged:: 0.5
           If the key does no longer exist a list is returned instead of
           raising an error.
        """
        return dict.pop(self, key, [])

    def popitemlist(self):
        """Pop a ``(key, list)`` tuple from the dict."""
        try:
            return dict.popitem(self)
        except KeyError, e:
            raise self.KeyError(str(e))

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.items(multi=True))


class ImmutableDictMixin(object):
    """Makes a :class:`dict` immutable.

    .. versionadded:: 0.5

    :private:
    """

    def __reduce_ex__(self, protocol):
        return type(self), (dict(self),)

    def setdefault(self, key, default=None):
        is_immutable(self)

    def update(self, *args, **kwargs):
        is_immutable(self)

    def pop(self, key, default=None):
        is_immutable(self)

    def popitem(self):
        is_immutable(self)

    def __setitem__(self, key, value):
        is_immutable(self)

    def __delitem__(self, key):
        is_immutable(self)

    def clear(self):
        is_immutable(self)


class ImmutableMultiDictMixin(ImmutableDictMixin):
    """Makes a :class:`MultiDict` immutable.

    .. versionadded:: 0.5

    :private:
    """

    def __reduce_ex__(self, protocol):
        return type(self), (self.items(multi=True),)

    def add(self, key, value):
        is_immutable(self)

    def popitemlist(self):
        is_immutable(self)

    def poplist(self, key):
        is_immutable(self)

    def setlist(self, key, new_list):
        is_immutable(self)

    def setlistdefault(self, key, default_list=None):
        is_immutable(self)


class ImmutableMultiDict(ImmutableMultiDictMixin, MultiDict):
    """An immutable :class:`MultiDict`.

    .. versionadded:: 0.5
    """

    def copy(self):
        """Return a shallow mutable copy of this object.  Keep in mind that
        the standard library's :func:`copy` function is a no-op for this class
        like for any other python immutable type (eg: :class:`tuple`).
        """
        return MultiDict(self)

    def __copy__(self):
        return self




'''
COM Bridge for SOAP Services
============================

This soaplib extension provides a simple COM bridge for accessing SOAP web
services, especially soaplib web services, via a COM-compliant language.  It
was developed primarily for use with standard Windows VBScript/VBA.

To use the COM bridge to access a web service, you must follow the following
steps:

    1. Register the COM bridge with Windows by executing this module from 
       the command-line:

            python soaplib\ext\comproxy.py
       
       You should see output stating that the object was registered.

    2. Place service stubs for the web service onto the PYTHONPATH so that
       soaplib can create a service client for that web service.  This is
       usually as easy as creating a soaplib web service object with empty
       methods that describes the web service you want to access, along with
       any special types.  For example:

            class Title(ClassSerializer):
                class types:
                    titleID     = Integer
                    name        = String
                    description = String

            class Person(ClassSerializer):
                class types:
                    personID    = Integer
                    firstName   = String
                    lastName    = String
                    birthdate   = DateTime
                    titles      = Array(Title)
            
            class PeopleService(SimpleWSGISoapApp): 
                
                @soapmethod(Person, _returns=Integer)
                def addPerson(self, person): pass

                @soapmethod(Integer, _returns=Person)
                def getPerson(self, personID): pass
    
    3. From your COM-compliant language, create an instance of the soaplib
       client object, then tell it about your web service:

            Set client = CreateObject("SoapLib.ServiceClient")
            
            uri = "http://webservicehost:port/"
            service_import_path = "services.people.PeopleService"
            
            client.SetServiceInfo uri, service_import_path
    
    4. Once you have a client object instantiated, you can use it to create
       instances of any complex types, and call remote methods:
            
            ' instantiate a person object and two title objects
            Set person = client.CreateObject("services.people.Person")
            Set titleOne = client.CreateObject("services.people.Title")
            Set titleTwo = client.CreateObject("services.people.Title")
            
            ' set some attributes on the first title
            titleOne.name = "Team Lead"
            titleOne.description = "Development Team Leader"
            
            ' set some attribtues on the second title
            titleTwo.name = "Smart Guy"
            titleTwo.description = "All-Around Smart Guy"
            
            ' set some attributes on the person, including a date/time
            ' and an Array of complex types
            person.firstName = "Jonathan"
            person.lastName = "LaCour"
            person.birthdate = Now()
            person.titles = Array(titleOne, titleTwo)
            
            ' call the web service to add this person to the database
            personID = client.addPerson(person)
            
            ' fetch the person back again, using the ID
            Set theperson = client.getPerson(personID)
            
            ' echo the results
            WScript.Echo "Retrieved person: " & theperson.personID
            WScript.Echo "First name: " & theperson.firstName
            WScript.Echo "Last name: " & theperson.lastName
            WScript.Echo "Birthdate: " & theperson.birthdate
            
            titles = theperson.titles
            For i = 0 to UBound(titles)
                Set title = titles(i)
                WScript.Echo "Title: " & title.name & ": " & title.description
            Next

In the future, we would like to make this easier to use by being able to just 
pass the URI to the WSDL for the service into the client, rather than having
to create Python stubs.
'''

from warnings import warn
warn('This module is under active development and should not be used in a production scenario')

from win32com.server.exception      import COMException
from win32com.server                import util
from soaplib.client                 import make_service_client
from soaplib.serializers.clazz      import ClassSerializer
from soaplib.serializers.primitive  import DateTime
from datetime                       import datetime

import winerror
import types
import time

# COM object wrapping and unwrapping utility functions
def coerce_date_time(dt):
    return datetime(*time.strptime(str(dt), '%m/%d/%y %H:%M:%S')[0:5])

def unwrap_complex_type(param, param_type):
    param = util.unwrap(param)
    for membername, membertype in param_type.soap_members.items():
        member = getattr(param, membername)
        if type(member).__name__ == 'PyIDispatch': 
            member = unwrap_complex_type(member, membertype)
        elif membertype is DateTime:
            member = coerce_date_time(member)
        elif type(member) in [types.ListType, types.TupleType]:
            newmember = []
            for item in member:
                if type(item).__name__ == 'PyIDispatch':
                    item = unwrap_complex_type(item, membertype.serializer)
                newmember.append(item)
            member = newmember
        setattr(param, membername, member)
    return param

def wrap_complex_type(data, data_type):
    for membername, membertype in data_type.soap_members.items():
        member = getattr(data, membername)
        if isinstance(member, ClassSerializer):
            member = wrap_complex_type(member, membertype)
        elif type(member) in [types.ListType, types.TupleType]:
            newmember = []
            for item in member:
                if isinstance(item, ClassSerializer):
                    item = wrap_complex_type(item, item.__class__)
                newmember.append(item)
            member = newmember
        setattr(data, membername, member)
    data = util.wrap(data)
    return data


class WebServiceClient:
    _reg_progid_        = 'SoapLib.ServiceClient'
    _reg_clsid_         = '{BAC77389-8687-4A8A-9DD0-2E4409FEF900}'
    _reg_policy_spec_   = 'DynamicPolicy'
    
    def SetServiceInfo(self, serviceURI, serviceName):
        try:
            parts = serviceName.split('.')
            item = __import__('.'.join(parts[:-1]))
            for part in parts[1:]:
                item = getattr(item, part)
            
            self.client_type = item()
            self.client = make_service_client(str(serviceURI), self.client_type)
        except:
            raise COMException('No such service', winerror.DISP_E_BADVARTYPE)
    
    def CreateObject(self, typename):
        try:
            parts = typename.split('.')
            item = __import__('.'.join(parts[:-1]))
            for part in parts[1:]:
                item = getattr(item, part)
            return util.wrap(item())
        except:
            raise COMException('No such type', winerror.DISP_E_BADVARTYPE)
    
    def _dynamic_(self, name, lcid, wFlags, args):
        # Look up the requested method.  First, check to see if the
        # method is present on ourself (utility functions), then
        # check to see if it exists on the client service.
        is_service_method = False
        item = getattr(self, name, None)
        if item is None and hasattr(self, 'client'):
            item = getattr(self.client, name)
            is_service_method = True
        
        if item is None:
            raise COMException('No attribute of that name.',
                                winerror.DISP_E_MEMBERNOTFOUND)
        
        # Figure out what parameters this web service call accepts,
        # and what it returns, so that we can properly wrap the objects
        # on the way in and unwrap them on the way out.
        if is_service_method:
            all_methods = self.client_type.methods()
            method_descriptor = [method for method in all_methods if method.name == name][0]
            return_type = method_descriptor.outMessage.params[0][1]
            parameter_types = [parameter[1] for parameter in method_descriptor.inMessage.params]

            # Now that we have this data, go ahead and unwrap any wrapped parameters,
            # recursively.
            newargs = []
            for param_type, param in zip(parameter_types, args):
                if hasattr(param_type, '__bases__') and ClassSerializer in param_type.__bases__:
                    param = unwrap_complex_type(param, param_type)
                elif param_type is DateTime:
                    param = coerce_date_time(param) 
                newargs.append(param)
            args = newargs
        
        # Call the supplied method
        result = apply(item, args)
        
        # Now wrap the return value, recursively.
        if isinstance(result, ClassSerializer):
            result = wrap_complex_type(result, return_type)
        
        # Return our data
        return result


if __name__ == '__main__':
    import win32com.server.register
    win32com.server.register.UseCommandLine(WebServiceClient)




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from lxml import etree

from soaplib.util.odict import odict

def root_dict_to_etree(d):
    assert len(d) == 1

    retval = etree.Element(d.keys()[0])
    for val in d.values():
        break

    if isinstance(val, dict) or isinstance(val, odict):
        dict_to_etree(retval, val)
    else:
        for a in val:
            dict_to_etree(retval, a)

    return retval

def dict_to_etree(parent, d):
    """the dict values are either dicts or iterables"""

    for k, v in d.items():
        if len(v) == 0:
            etree.SubElement(parent,k)

        elif isinstance(v, dict) or isinstance(v, odict):
            child = etree.SubElement(parent,k)
            dict_to_etree(child,v)

        else:
            for e in v:
                child=etree.SubElement(parent,k)
                if isinstance(e, dict) or isinstance(e, odict):
                    dict_to_etree(child,e)
                else:
                    child.text=str(e)

def root_etree_to_dict(element, iterable=(list, list.append)):
    return {element.tag: [etree_to_dict(element)]}

def etree_to_dict(element, iterable=(list,list.append)):
    if (element.text is None) or element.text.isspace():
        retval = odict()
        for elt in element:
            if not (elt.tag in retval):
                retval[elt.tag] = iterable[0]()
            iterable[1](retval[elt.tag], etree_to_dict(elt, iterable))

    else:
        retval = element.text

    return retval



from soaplib.etimport import ElementTree

import httplib
from soaplib.soap import from_soap, make_soap_envelope, collapse_swa, apply_mtom
from soaplib.util import split_url, create_relates_to_header
from soaplib.serializers.primitive import Fault
from cStringIO import StringIO

import sys

# This sets the HTTP version string sent to the server to 1.0
# preventing the response from bein 'chunked'.  This is done
# because of a know bug in python (#900744).  Rather than apply 
# the patch to all the installed systems, it is simpler to set this
# version string, to be later removed in python 2.5
#
httplib.HTTPConnection._http_vsn_str = 'HTTP/1.0'

_debug = False
_out = sys.stdout

def debug(is_on,out=sys.stdout):
    '''
    This is a utility method for debugging client request and responses
    @param boolean for turning debug on or off
    @param filelike object to write to
    '''
    global _out, _debug
    _out = out
    _debug = is_on

def dump(host,path,headers,envelope):
    '''
    Debugging method for dumping request information to a file or stdout
    @param host server host
    @param path server path
    @param headers http headers
    @param envelope soap envelope
    '''
    global _out, _debug
    if not _debug: return

    def writeln(text):
        _out.write(text)
        _out.write('\r\n')
        _out.flush()
        
    writeln('-------------------------------------------------')
    writeln('Host: '+host)
    writeln('Path: '+path)
    writeln('Headers:')
    for k,v in headers.items():
        writeln('    %s -> %s'%(k.ljust(20),v))
    writeln('Envelope:')
    writeln(envelope)
    writeln('-------------------------------------------------')
    
_err_format =     '''
Parameter Do Not Match: 
+ Arguments Passed In: 
%s 
+ Parameters Required: 
%s'''

class SimpleSoapClient(object):
    '''
    SimpleSoapClient is the lowest level http soap client in soaplib,
    and represents a single remote soap method.  This class can be 
    used by itself by passing it the url information and MethodDescriptor, 
    but is most frequently used by the ServiceClient object.
    '''

    def __init__(self,host,path,descriptor,scheme="http"):
        '''
        @param remote host
        @param remote path
        @param MethodDescriptor of the remote method being called
        @param remote scheme
        '''
        self.host = host
        self.path = path
        self.descriptor = descriptor
        self.scheme = scheme

    def __call__(self,*args,**kwargs):
        '''
        This method executes the http request to the remote web service.  With
        the exception of 'headers', 'msgid', and 'mtom'; all keyword arguments 
        to this method are put in the http header.  The 'headers' keyword is to
        denote a list of elements to be included in the soap header, 'msgid'
        is a convenience keyword used in async web services which creates a
        WS-Addressing messageid header to be included in the soap headers, and
        'mtom' enables the Message Transmission Optimization Mechanism.

        @param the arguments to the remote method
        @param the keyword arguments 
        '''
        if len(args) != len(self.descriptor.inMessage.params):
            argstring = '\r\n'.join(['    '+str(arg) for arg in args])
            paramstring = '\r\n'.join(['    '+str(p[0]) for p in self.descriptor.inMessage.params])
            err_msg = _err_format%(argstring,paramstring)
            raise Exception(err_msg)
        
        msg = self.descriptor.inMessage.to_xml(*args)

        # grab the soap headers passed into this call
        headers = kwargs.get('headers',[])
        mtom = kwargs.get('mtom',False)
        msgid = kwargs.get('msgid')
        if msgid:
            # special case for the msgid field as a convenience 
            # when dealing with async callback methods
            headers.append(create_relates_to_header(msgid))

        tns = self.descriptor.inMessage.ns
        envelope = make_soap_envelope(msg, tns, header_elements=headers)

        body = ElementTree.tostring(envelope)
        methodName = '\"%s\"'%self.descriptor.soapAction
        httpHeaders = {"Content-Length":len(body),
                      "Content-type":'text/xml; charset="UTF-8"',
                      "Accept":"application/soap+xml, application/dime, multipart/related, text/*",
                      'User-Agent':'Soaplib/1.0',
                      'SOAPAction':methodName
                      }
                      
        for k,v in kwargs.items():
            # add all the other keywords to the http headers
            if k not in ('headers','msgid','mtom'):
                httpHeaders[k]=v

        if mtom:
            httpHeaders, body = apply_mtom( httpHeaders, body,
                                            self.descriptor.inMessage.params,
                                            args )
 
        dump(self.host,self.path,httpHeaders,body)               

        if self.scheme == "http": 
            conn = httplib.HTTPConnection(self.host)
        elif self.scheme == "https":
            conn = httplib.HTTPSConnection(self.host)
        else:
            raise RuntimeError("Unsupported URI connection scheme: %s" % scheme)
            
        conn.request("POST",self.path,body=body,headers=httpHeaders)
        response = conn.getresponse()
        data = response.read() 
        
        dump(self.host,self.path,dict(response.getheaders()),data)
        
        contenttype = response.getheader('Content-Type')
        data = collapse_swa(contenttype, data)
        
        conn.close()
        if str(response.status) not in['200','202']:
            # consider everything NOT 200 or 202 as an error response
            
            if str(response.status) == '500': 
                fault = None
                try:
                    payload, headers = from_soap(data)
                    fault = Fault.from_xml(payload)
                except:
                    trace = StringIO()
                    import traceback
                    traceback.print_exc(file=trace)
                    
                    fault = Exception('Unable to read response \n  %s %s \n %s \n %s'%(response.status,response.reason,trace.getvalue(),data))
                raise fault
            else:
                raise Exception('%s %s'%(response.status,response.reason))

        if not self.descriptor.outMessage.params:
            return 

        payload, headers = from_soap(data)
        results = self.descriptor.outMessage.from_xml(payload)
        return results[0] 

class ServiceClient(object):
    '''
    This class is a simple, convenient class for calling remote web services.
    @param host the host of the SOAP service being called
    @param path the path to the web service
    @param impl the SimpleWSGISoapApp which defines the remote service
    @param mtom whether or not to send attachments using MTOM
    '''

    def __init__(self,host,path,server_impl, scheme="http"):
        if host.startswith("http://"):
            host = host[6:]
            
        self.server = server_impl
        for method in self.server.methods():
            setattr(self,method.name,SimpleSoapClient(host,path,method,scheme))

def make_service_client(url,impl):
    scheme,host,path = split_url(url)
    return ServiceClient(host,path,impl, scheme)




from lxml import etree

from base import SimpleType
from base import nillable_element
from base import nillable_value
from base import string_to_xml

import soaplib

_ns_xs = soaplib.ns_xsd

# adapted from: http://code.activestate.com/recipes/413486/

class EnumBase(SimpleType):
    __namespace__ = None

    @staticmethod
    def resolve_namespace(cls, default_ns):
        if cls.__namespace__ is None:
            cls.__namespace__ = default_ns

    @classmethod
    @nillable_value
    def to_xml(cls, value, tns, parent_elt, name='retval'):
        if name is None:
            name = cls.get_type_name()

        string_to_xml(cls, str(value), tns, parent_elt, name)

    @classmethod
    @nillable_element
    def from_xml(cls, element):
        return getattr(cls, element.text)

def Enum(*values, **kwargs):
    type_name = kwargs.get('type_name', None)
    if type_name is None:
        raise ValueError("Please specify 'type_name' as a keyword argument")

    assert len(values) > 0, "Empty enums are meaningless"

    maximum = len(values) # to make __invert__ work

    class EnumValue(object):
        __slots__ = ('__value')

        def __init__(self, value):
            self.__value = value

        def __hash__(self):
            return hash(self.__value)

        def __cmp__(self, other):
            assert type(self) is type(other), \
                             "Only values from the same enum are comparable"

            return cmp(self.__value, other.__value)

        def __invert__(self):
            return values[maximum - self.__value]

        def __nonzero__(self):
            return bool(self.__value)

        def __repr__(self):
            return str(values[self.__value])

    class EnumType(EnumBase):
        __type_name__ = type_name

        def __iter__(self):
            return iter(values)

        def __len__(self):
            return len(values)

        def __getitem__(self, i):
            return values[i]

        def __repr__(self):
            return 'Enum' + str(enumerate(values))

        def __str__(self):
            return 'enum ' + str(values)

        @classmethod
        def add_to_schema(cls, schema_entries):
            if not schema_entries.has_class(cls):
                simple_type = etree.Element('{%s}simpleType' % _ns_xs)
                simple_type.set('name', cls.get_type_name())

                restriction = etree.SubElement(simple_type,
                                                    '{%s}restriction' % _ns_xs)
                restriction.set('base', '%s:string' %
                                   soaplib.get_namespace_prefix(soaplib.ns_xsd))

                for v in values:
                    enumeration = etree.SubElement(restriction,
                                                    '{%s}enumeration' % _ns_xs)
                    enumeration.set('value', v)

                schema_entries.add_simple_type(cls, simple_type)

    for i,v in enumerate(values):
        setattr(EnumType, v, EnumValue(i))

    return EnumType



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

__all__ = ["get_modules", "WebApplication", "WSGIApplication", "DebugApplication", "make_application"]

import os
import re
import logging
import tornado.options
from tornado.web import (Application, RequestHandler, StaticFileHandler as _StaticFileHandler,
                         RedirectHandler, HTTPError, URLSpec)
import tornado.wsgi
from tornado.wsgi import WSGIApplication as _WSGIApplication
from torweb.urls import url
from torweb.config import Yaml_Config
#from torweb.handlers import StaticFileHandler, ErrorHandler, WSGIRequest
from torweb.handlers import StaticFileHandler, ErrorHandler
from torweb import base_path
from code import interact


def get_modules(prefix, parent):
    if not os.path.isfile(os.path.join(parent, '__init__.py')):
        return
    for pathname in os.listdir(parent):
        child = os.path.join(parent, pathname)
        if pathname.startswith('.'):
            continue
        elif pathname == '__init__.py':
            yield '.'.join(prefix), pathname.split('.')[0]
        elif os.path.isdir(child):
            for i in get_modules(prefix+[pathname], child):
                yield i, pathname.split('.')[0]
        elif os.path.isfile(child) and pathname.endswith('.py'):
            yield '.'.join(prefix+[pathname[:-3]]), pathname.split('.')[0]


class WebApplication(Application):
    def __call__(self, request): 
        #print """Called by HTTPServer to execute the request."""
        #print "##################################################"
        if not hasattr(self, "add_slash"):
            setattr(self, "add_slash", False)
        _add_slash = getattr(self, "add_slash")
        transforms = [t(request) for t in self.transforms]
        handler = None
        args = []
        kwargs = {}
        if _add_slash == False:
            # removeslash
            if request.path.endswith("/"):
                if request.method in ("GET", "HEAD"):
                    uri = request.path.rstrip("/")
                    if uri:  # don't try to redirect '/' to ''
                        if request.query:
                            uri += "?" + request.query
                        hdr = RedirectHandler(self, request, url=uri)
                        hdr._execute(transforms, *args, **kwargs)
                        return hdr
                else:
                    raise HTTPError(404)
        else:
            # addslash
            li = [_compile for _compile in self._static_urls if re.compile(_compile).match(request.path)]
            if len(li) == 0 and not request.path.endswith("/"):
                if request.method in ("GET", "HEAD"):
                    uri = request.path + "/"
                    if request.query:
                        uri += "?" + request.query
                    hdr = RedirectHandler(self, request, url=uri)
                    hdr._execute(transforms, *args, **kwargs)
                    return hdr
                raise HTTPError(404)
        super(WebApplication, self).__call__(request)


class WSGIApplication(_WSGIApplication):
    def __call__(self, environ):
        handler = web.Application.__call__(self, tornado.wsgi.HTTPRequest(environ))
        assert handler._finished
        status = str(handler._status_code) + " " + \
            httplib.responses[handler._status_code]
        headers = handler._headers.items()
        if hasattr(handler, "_new_cookie"):
            for cookie in handler._new_cookie.values():
                headers.append(("Set-Cookie", cookie.OutputString(None)))
        start_response(status,
                       [(native_str(k), native_str(v)) for (k, v) in headers])
        return handler._write_buffer


class DebugApplication(WebApplication):
    "Tornado Application supporting werkzeug interactive debugger."
 
    # This supports get_error_html in Handler above.
 
    def __init__(self, *args, **kwargs):
        from werkzeug.debug import DebuggedApplication
        self.debug_app = DebuggedApplication(self.debug_wsgi_app, evalex=True)
        self.debug_container = tornado.wsgi.WSGIContainer(self.debug_app)
        super(DebugApplication, self).__init__(*args, **kwargs)
 
    def __call__(self, request):
        if '__debugger__' in request.uri:
            # Do not call get_current_traceback here, as this is a follow-up
            # request from the debugger. DebugHandler loads the traceback.
            return self.debug_container(request)
        return super(DebugApplication, self).__call__(request)
 
    @classmethod
    def debug_wsgi_app(cls, environ, start_response):
        print("Fallback WSGI application, wrapped by werkzeug's debug middleware.")
        status = '500 Internal Server Error'
        response_headers = [('Content-type', 'text/plain')]
        start_response(status, response_headers)
        return ['Failed to load debugger.\n']
 
    def get_current_traceback(self):
        "Get the current Python traceback, keeping stack frames in debug app."
        traceback = get_current_traceback()
        for frame in traceback.frames:
            self.debug_app.frames[frame.id] = frame
        self.debug_app.tracebacks[traceback.id] = traceback
        return traceback
 
    def get_traceback_renderer_keywords(self):
        "Keep consistent debug app configuration."
        # DebuggedApplication generates a secret for use in interactions.
        # Otherwise, an attacker could inject code into our application.
        # Debugger gives an empty response when secret is not provided.
        return dict(evalex=self.debug_app.evalex, secret=self.debug_app.secret)

def get_current_traceback():
    "Get the current traceback in debug mode, using werkzeug debug tools."
    # Lazy import statement, as debugger is only used in development.
    from werkzeug.debug.tbtools import get_current_traceback
    # Experiment with skip argument, to skip stack frames in traceback.
    traceback = get_current_traceback(skip=2, show_hidden_frames=False,
                                      ignore_system_exceptions=True)
    return traceback

 
def make_application(app, debug=False, wsgi=False, settings_path="", url_root="/", **keywords):
    app_path = os.path.abspath(os.path.dirname(app.__file__))
    conf = None
    if settings_path != "":
        #conf = app.settings
        conf = Yaml_Config(app_path, settings_path, url_root=url_root)
        setattr(app, 'yaml_conf', conf)
        #logging.info(url_root)
    #else:
    #    conf = Yaml_Config(app_path, os.path.join(app_path, 'settings.yaml'), url_root=url_root)
    views_path = os.path.join(app_path, 'views')
    module_list = []
    def getModule(module):
        if type(module) == tuple:
            return getModule(module[0])
        else:
            return module

    for module, name in get_modules([app.__name__, 'views'], views_path):
        module = getModule(module)
        #if type(module) == tuple:
        #    module = module[0]
        #    if type(module) == tuple:
        #        module = module[0]
        m = __import__(module, {}, {}, name)
        module_list.append(m)
    from torweb.urls import url_rules, except_url, url404
    def _set_debug(kw):
        kw["debug"] = debug
        return kw
    url_handlers = []
    url_rules.extend(url404.handlers)
    app_url_handlers = url_rules
    url_handlers.extend(app_url_handlers)
    url_handlers.extend(except_url.handlers)
    url_handlers = [URLSpec(spec.regex.pattern, spec.handler_class, _set_debug(spec.kwargs), spec.name) for spec in url_handlers]
    _static_urls = [spec.regex.pattern for spec in url.handlers if spec.kwargs.has_key("path") and hasattr(spec.handler_class, 'static_handler') and spec.handler_class.static_handler==True]

    #for _uri in url_handlers:
    #    logging.info("%s <name:%s>" % (_uri.regex.pattern, _uri.name))

    keywords.update({"debug":debug})
    if debug:
        application = DebugApplication(url_handlers, **keywords)
    else:
        if wsgi == True:
            application = _WSGIApplication(url_handlers, **keywords)
        else:
            application = WebApplication(url_handlers, **keywords)
    application.url_handlers = url_handlers
    application._static_urls = _static_urls
    # 初始化session store
    if conf:
        application.session_store = conf.session_store
    else:
        from torweb.sessions import MemorySessionStore
        application.session_store = MemorySessionStore()
    return application

class InitializeApplication(object):
    def __init__(self, app):
        self.app = app
        self.app_path = os.path.abspath(os.path.dirname(app.__file__))
        self.conf = Yaml_Config(os.path.join(app_path, 'settings.yaml'))
        pass
    def make_application(self, debug, wsgi):
        return make_application(self.app, debug, wsgi)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

import os
import re
import urlparse
import logging
from tornado.web import URLSpec
from code import interact

url_rules = []
class Url(object):
    def __init__(self, prefix="/", except_url=False, status_code=200):
        self.prefix = prefix
        self.handlers = []
        self.status_code = status_code

    def __call__(self, url, **kwds):
        def _(cls):
            kwname = kwds.get("name", None)
            prefix = kwds.get("prefix") or self.prefix
            status_code = self.status_code
            name = kwname or "%s.%s" % (cls.__module__, cls.__name__)
            if url == "/":
                _url = prefix
            else:
                #_url = prefix + "/" + re.sub("/{0,1}", "", url, 1)
                #_url = "/".join([prefix, re.sub("/{0,1}", "", url, 1)])
                if not prefix.endswith("/"):
                    prefix = prefix + "/"
                _url = urlparse.urljoin(prefix, re.sub("/{0,1}", "", url, 1))
            self.handlers.append(URLSpec(_url, cls, kwds, name=name))
            #logging.info("[status_code-->%s][url:%s]" % (status_code, _url))
            if status_code == 200:
                url_rules.append(URLSpec(_url, cls, kwds, name=name))
            setattr(cls, "url_pattern", _url)
            return cls
        return _
url = Url()
except_url = Url()
url404 = Url(status_code=404)



import httplib
import datetime
import urllib
from urllib import quote
from soaplib.etimport import ElementTree

def create_relates_to_header(relatesTo,attrs={}):
    '''Creates a 'relatesTo' header for async callbacks'''
    relatesToElement = ElementTree.Element('{http://schemas.xmlsoap.org/ws/2003/03/addressing}RelatesTo')
    for k,v in attrs.items():
        relatesToElement.set(k,v)
    relatesToElement.text = relatesTo
    return relatesToElement

def create_callback_info_headers(messageId,replyTo):
    '''Creates MessageId and ReplyTo headers for initiating an async function'''
    messageIdElement = ElementTree.Element('{http://schemas.xmlsoap.org/ws/2003/03/addressing}MessageID')
    messageIdElement.text = messageId

    replyToElement = ElementTree.Element('{http://schemas.xmlsoap.org/ws/2003/03/addressing}ReplyTo')
    addressElement = ElementTree.SubElement(replyToElement,'{http://schemas.xmlsoap.org/ws/2003/03/addressing}Address')
    addressElement.text = replyTo
    return messageIdElement, replyToElement

def get_callback_info():
    '''Retrieves the messageId and replyToAddress from the message header.
    This is used for async calls.'''
    messageId = None
    replyToAddress = None
    from soaplib.wsgi_soap import request
    headerElement = request.header
    if headerElement:
        headers = headerElement.getchildren()
        for header in headers:
            if header.tag.lower().endswith("messageid"):
                messageId = header.text
            if header.tag.lower().find("replyto") != -1:
                replyToElems = header.getchildren()
                for replyTo in replyToElems:
                    if replyTo.tag.lower().endswith("address"):
                        replyToAddress = replyTo.text
    return messageId, replyToAddress

def get_relates_to_info():
    '''Retrives the relatesTo header. This is used for callbacks'''
    from soaplib.wsgi_soap import request

    headerElement = request.header
    if headerElement:
        headers = headerElement.getchildren()
        for header in headers:
            if header.tag.lower().find('relatesto') != -1:
                return header.text

def split_url(url):
    '''Splits a url into (uri_scheme, host[:port], path)'''
    scheme, remainder = urllib.splittype(url)
    host, path = urllib.splithost(remainder)
    return scheme.lower(), host, path

def reconstruct_url(environ):
    '''
    Rebuilds the calling url from values found in the
    environment.

    This algorithm was found via PEP 333, the wsgi spec and
    contributed by Ian Bicking.
    '''
    url = environ['wsgi.url_scheme']+'://'
    if environ.get('HTTP_HOST'):
        url += environ['HTTP_HOST']
    else:
        url += environ['SERVER_NAME']

        if environ['wsgi.url_scheme'] == 'https':
            if environ['SERVER_PORT'] != '443':
               url += ':' + environ['SERVER_PORT']
        else:
            if environ['SERVER_PORT'] != '80':
               url += ':' + environ['SERVER_PORT']

    if quote(environ.get('SCRIPT_NAME','')) == '/' and quote(environ.get('PATH_INFO',''))[0:1] == '/':
        #skip this if it is only a slash
        pass
    elif quote(environ.get('SCRIPT_NAME',''))[0:2] == '//':
        url += quote(environ.get('SCRIPT_NAME',''))[1:]
    else:
        url += quote(environ.get('SCRIPT_NAME',''))

    url += quote(environ.get('PATH_INFO',''))
    if environ.get('QUERY_STRING'):
        url += '?' + environ['QUERY_STRING']
    return url
    

###################################################################
# Deprecated Functionality
###################################################################
from warnings import warn
def deprecate(name):
    warn("Method [%s] will be removed at the end of this iteration"%name,DeprecationWarning)    

def convertDateTime(date):
    deprecate('convertDateTime')
    date = date.replace("T"," ")
    d, t = date.split(' ')
    y,mo,da = d.split('-')
    h,mi,s = t.split(':')
    ms = 0
    try: s,ms = s.split('.')
    except: pass
    return datetime.datetime(int(y),int(mo),int(da),int(h),int(mi),int(s),int(ms))

converters = {
    'datetime':convertDateTime,
    'integer':int,
    'float':float,
    'boolean':bool,
}

def element2dict(element):
    deprecate('element2dict')
    if type(element) == str:
        element = ElementTree.fromstring(element)

    children = element.getchildren()
    tag = element.tag.split('}')[-1] 
    return {tag:_element2dict(children)}

def _get_element_value(element):
    deprecate('_get_element_value')
    xsd_type = None
    for k in element.keys():
        if k.lower().endswith('type'):
            xsd_type = element.get(k)
    if element.text == None:
        return None
    if xsd_type:
        t = xsd_type.lower().split(':')[-1]
        conv = converters.get(t)
        if conv: return conv(element.text)
        else: return element.text
    return element.text

def _element2dict(child_elements):
    deprecate('_element2dict')
    d = {}
    for child in child_elements:

        tag = child.tag.split('}')[-1] 
        children = child.getchildren()
        if children:
            typ = None
            for k in child.keys():
                if k.lower().endswith('type'):
                    typ = child.get(k)
            if typ and typ.lower().endswith('array'):
                d[tag] = []
                for c in child.getchildren():
                    if c.getchildren():
                        d[tag].append(_element2dict(c.getchildren()))
                    else:
                        d[tag].append(_get_element_value(c))
            else:
                d[tag] = _element2dict(children) 
        else:
            typ = None
            for k in child.keys():
                if k.lower().endswith('type'):
                    typ = child.get(k)
            value = _get_element_value(child)
            d[tag] = _get_element_value(child)
    return d


def dict2element(*args,**kwargs):
    deprecate('dict2element')
    if len(kwargs) == 1:
        dictionary = kwargs
    else:
        dictionary = args[0]
    if not len(dictionary.keys()):
        return ElementTree.Element('none')
    root = dictionary.keys()[0] 
    element =  _dict2element(dictionary[root],root)
    element.set('xmlns:optio','http://www.optio.com/schemas')
    return element

def _dict2element(data,tag):
    deprecate('_dict2element')
    d = {   datetime.datetime:'xs:dateTime',
            int:'xs:integer',
            bool:'xs:boolean',
            float:'xs:float',
           } 
    root = ElementTree.Element(tag)
    if type(data) == dict:
        for k,v in data.items():
            root.append(_dict2element(v,k))
    elif type(data) == list or type(data) == tuple:
        root.set('type','optio:array')
        for item in data:
            root.append(_dict2element(item,'item'))
    elif data is not None:
        t = d.get(type(data),'xs:string')
        root.text = str(data)
        root.set('type',t)
    return root




import urllib2
import new
from soaplib.soap import Message, MethodDescriptor
from soaplib.serializers import primitive
from soaplib.serializers.clazz import ClassSerializer
from soaplib.client import *
from soaplib.etimport import ElementTree as et

from warnings import warn
warn('This module is under active development and should not be used in a production scenario')


_primitives = {
    'xs:string':primitive.String,
    'xs:int':primitive.Integer,
    'xs:dateTime':primitive.DateTime,
    'xs:float':primitive.Float,
    'xs:boolean':primitive.Boolean,
    'string':primitive.String,
    'int':primitive.Integer,
    'dateTime':primitive.DateTime,
    'float':primitive.Float,
    'boolean':primitive.Boolean
}

_types = {}
_messages = {}
_methods = {}
_clients = {}
_service_name = ''
_location = ''

wsdl = 'http://schemas.xmlsoap.org/wsdl/'
xs = 'http://www.w3.org/2001/XMLSchema'

_serializers = {}
ns = 'test'

_type_elements = {}
    
def is_array(name):
    # need better checking for arrays
    #print _type_elements
    print _types
    if len(_type_elements[name]) == 1 and _type_elements[name][0].get('maxOccurs') == 'unbounded':
        return True
    return False

def get_serializer(name):
    print name
    print _types
    if _primitives.has_key(name):
        return _primitives[name]
        
    if not _serializers.has_key(name):
        name = name.split(':')[-1]
        
        if is_array(name):
            s = _types[name]
            inner_serializer = get_serializer( s.keys()[0] )
            class_serializer = primitive.Array(inner_serializer)
        else:
            s = _types[name]
            print s
            class_serializer = new.classobj(name,(ClassSerializer,),{})
            setattr(class_serializer,'soap_members',{})
            setattr(class_serializer,'namespace',None)
            for k,v in s.items():
                class_serializer.soap_members[k] = get_serializer(v)
        _serializers[name] = class_serializer
        
    return _serializers[name]

def qn(prefix,typ):
    return '{%s}%s'%(globals()[prefix],typ)

def handle_wsdl(wsdl):
    handle_types(wsdl.find(qn('wsdl','types')))
    handle_messages(wsdl.findall(qn('wsdl','message')))
    handle_portType(wsdl.find(qn('wsdl','portType')))
    handle_service(wsdl.find(qn('wsdl','service')))
    print _types
    
def handle_service(service):
    _service_name = service.get('name')
    location = service.getchildren()[0].getchildren()[0].get('location')
    _location = location
    
def handle_types(types):
    schema = types[0]
    elements = schema.findall(qn('xs','element'))
    for element in elements:
        for c in schema.findall(qn('xs','complexType')):
            if c.get('name') == element.get('name'):
                handle_element(element,c)
    for k in _types.keys():
        print get_serializer(k)
                        
def handle_element(element,complexType):
    parts = {}
    se = complexType.find(qn('xs','sequence')).findall(qn('xs','element'))
    name = element.get('name')
    _type_elements[name] = se
    print se
    for e in se:
        print 'adding',e.get('name')
        parts[e.get('name')] = e.get('type')
        #array = ''
        #if e.get('maxOccurs') == 'unbounded': # check for arrays?
        #    array = 'array'
        #print '     ',e.get('name'),e.get('type'), array
    print parts
    _types[name] = parts
    #_serializers = {}

def get_params(msg):
    print msg
    params = _types[msg]
    msg_params = []
    for k,v in params.items():
        if v.startswith('xs'):
            msg_params.append((k,_primitives[v]))
        else:
            print 'k,v',k,v
            v = v.split(':')[-1]
            msg_params.append((k,_serializers[v]))
            #print k,v
            #msg_params.append()
    return msg_params
    
def handle_messages(messages):
    print messages
    print "^"*80
    for m in messages:
        name = m.get('name')
        p = m.find(qn('wsdl','part'))
        _messages[name] = (p.get('element'),p.get('name'))
        params = get_params(name)
        print 'params',params
        
        msg_obj = Message(name,name,params)
        _messages[name] = msg_obj
        
def handle_portType(pt):
    operations = pt.findall(qn('wsdl','operation'))
    for operation in operations:
        name = operation.get('name')
        input = operation.find(qn('wsdl','input'))
        in_msg = input.get('message')
        in_name = input.get('name')
        
        output = operation.find(qn('wsdl','output'))
        if output != None:
            out_msg = output.get('message')
            out_name = output.get('name')
        else:
            out_msg = ''
            out_name = ''
               
        print '*'*80
        print _messages[in_msg.split(':')[-1]]
        print '  name  ',name
        print '  input ',in_msg,in_name
        print ' output ',out_msg,out_name
        
        method = MethodDescriptor(name,_messages[in_name],_messages[out_name])
        _methods[name] = method
    
def make_clients():
    print _service_name
    for name, method in _methods.items():
        _clients[name] = SimpleSoapClient('127.0.0.1:9753','/',method)
    service = new.classobj(_service_name,(object,),{})
    for operation, client in _clients.items():
        setattr(service,operation,client)
    return service
    
def run(url):
    content = urllib2.urlopen(url).read()
    wsdl = et.fromstring(content)
    handle_wsdl(wsdl)
    return make_clients()

if __name__ == "__main__":
    service = run('file:///c:/workspace/wsdl/ctcc_mm_receive_service_2_2.wsdl')
    #print _serializers
    #a = _serializers['echoSimpleClass']
    #print a
    #print dir(a)
    #print dir(a.soap_members)
    
    #print '*'*80
    #r = a()
    #r.sc = _serializers['SimpleClass']()
    #r.sc.i = 15
    #r.sc.s = 'af'
    #print _type_elements
    nc = _serializers['NestedClass']()
    nc.s = 'blah'
    #print dir(nc)
    simple = _serializers['SimpleClass']()
    simple.i = 13
    simple.s = 'ab'
    nc.simple = [simple]
    print service.echoNestedClass(nc).s
    print service.echoSimpleClass(simple).i



from soaplib.etimport import ElementTree

class NamespaceLookup(object):
    '''
    Class to manage XML namespaces
    '''

    def __init__(self, tns = None, wsdl_map = False):
        self.nsmap = {
            'xs': 'http://www.w3.org/2001/XMLSchema',
            'xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            #'plnk':'http://schemas.xmlsoap.org/ws/2003/05/partner-link/',
        }
        if wsdl_map:
            self.nsmap['soap'] = 'http://schemas.xmlsoap.org/wsdl/soap/'
            self.nsmap['wsdl'] = 'http://schemas.xmlsoap.org/wsdl/'
        else:
            self.nsmap['SOAP-ENC'] = 'http://schemas.xmlsoap.org/soap/encoding/'
            self.nsmap['SOAP-ENV'] = 'http://schemas.xmlsoap.org/soap/envelope/'
        if tns is not None:
            self.nsmap['tns'] = tns
            self.nsmap['typens'] = tns
        
    def get_all(self):
        '''
        Return all namespaces
        '''
        return self.nsmap
        
    def get(self, key):
        '''
        Lookup and return a given namespace
        '''
        ns = self.nsmap[key] if key in self.nsmap else ''
        return "{%s}" % ns
        
    def set(self, key, ns):
        '''
        Add a namespace to the map (replaces)
        '''
        self.nsmap[key] = ns

'''
Default namespace lookup
'''        
ns = NamespaceLookup()


def qualify(name, ns):
    '''
    Qualify an idenifier with a namespace
    '''
    return "{%s}%s" % (ns, name)


def create_xml_element(name, nslookup, default_ns=None):
    '''
    Factory method to create a new XML element
    @param default_ns The default namespace to use for the element.
    @param extended_map A mapping of any additional namespaces to add.
    '''
    namespace_map = { None: default_ns } if default_ns is not None else {}
    for key, value in nslookup.get_all().iteritems():
        if value != default_ns:
            namespace_map[key] = value
    return ElementTree.Element(name, nsmap=namespace_map)
    
    
def create_xml_subelement(parent, name):
    '''
    Factory method to create a new XML subelement
    '''
    if not name.startswith("{") and None in parent.nsmap:
        name = qualify(name, parent.nsmap[None])
    return ElementTree.SubElement(parent, name)
    



from soaplib.serializers.clazz import ClassSerializer
from soaplib.serializers.primitive import *
from soaplib.soap import *
from soaplib.util import split_url
from soaplib.etimport import ElementTree as et

import new, datetime, httplib

_builtin = {
    int : Integer,
    str : String,
    datetime.datetime : DateTime,
    float : Float,
    bool : Boolean,
}

class caller(object):
    def __init__(self,host,namespace,method):
        self.method = method
        self.host = host
        self.namespace = namespace
        
    def __call__(self,data,raw=False):
        e = data.to_xml(data,data.__class__.__name__)
        envelope = make_soap_envelope(e)
        body = et.tostring(envelope)
        methodName = '\"%s\"'%self.method
        httpHeaders = {"Content-Length":len(body),
                      "Content-type":'text/xml; charset="UTF-8"',
                      "Accept":"application/soap+xml, application/dime, multipart/related, text/*",
                      'User-Agent':'Soaplib/1.0',
                      'SOAPAction':methodName
                      }
        scheme,host,path = split_url(self.host)
        if scheme == "http":
            conn = httplib.HTTPConnection(host)
        elif scheme == "https":
            conn = httplib.HTTPSConnection(host)
        else:
            raise RuntimeError("Unsupported URI connection scheme: %s" % scheme)

        conn.request("POST",path,body=body,headers=httpHeaders)
        response = conn.getresponse()
        raw_data = response.read()
        
        retval = et.fromstring(raw_data)
        d = retval.find('{http://schemas.xmlsoap.org/soap/envelope/}Body').getchildren()[0]
        
        if raw:
            return d
        return objectify(d)

class DocumentClient(object):
   
    def __init__(self,host,methods,namespace=None):
        for method in methods:
            setattr(self,method,caller(host,namespace,method))
			
def get_serializer(value):
    _type = type(value)
    if value is None:
        return Null
    elif hasattr(value,'to_xml') and hasattr(value,'from_xml'):
        # this object can act as it's own serializer
        return value
    elif _builtin.has_key(_type):
        # found primitive: string, int, etc.
        return _builtin[_type]
    elif _type == list:
        # must assume that all the elements in the list are of the same type
        # and use the same serializer
        if not len(value):
            return Null
        else:
            return Array(get_serializer(value[0]))
    else:
        raise Exception("Could not find serializer for [%s]"%value)

def denamespace(tag):
    if tag.find('}') > -1:
        tag = tag.replace("{","")
        return tag.split('}')
    return None, tag
    
def objectify(element):
    class ElementWrapper(object):
        def __init__(self,element):
            self.__element = element
            self.__tags = {}
            for child in element.getchildren():
                ns,tag = denamespace(child.tag)
                if self.__tags.has_key(tag):
                    self.__tags[tag].append(child)
                else:
                    self.__tags[tag] = child
                    
                if hasattr(self,tag):
                    spot = getattr(self,tag)
                    if type(spot) != list:
                        spot = [spot]
                    #spot.append(objectify(child))
                    if len(child.getchildren()):
                        spot.append(new.classobj(tag,(ElementWrapper,),{})(child))
                    else:
                        spot.append(child.text)
                    setattr(self,tag,spot)
                    setattr(self,'__islist__',True)
                elif len(child.getchildren()):
                    setattr(self,tag,new.classobj(tag,(ElementWrapper,),{})(child))
                else:
                    setattr(self,denamespace(child.tag)[1],child.text) # marshall the type here!

        def __getitem__(self,index):
            if len(self.__tags.items()) == 1:
                # this *could* be an array
                k = self.__tags.keys()[0]
                return getattr(self,k)[index]
            if index == 0:
                return self
            raise StopIteration
            
    ns,tag = denamespace(element.tag)
    return new.classobj(tag,(ElementWrapper,),{})(element)
            
 
def make(__type_name,**kwargs):
    serializer = new.classobj(__type_name,(ClassSerializer,),{})
    namespace = None
	
    setattr(serializer,'soap_members',{})
    setattr(serializer,'namespace',namespace)
    for k,v in kwargs.items():
        serializer.soap_members[k] = get_serializer(v)
    o = serializer()
    for k,v in kwargs.items():
        setattr(o,k,v)
    return o
    
if __name__ == '__main__':
	echoInteger = make('echoInteger',i=34)
	print et.tostring(echoInteger.to_xml(echoInteger,'echoInteger'))
	
	c = DocumentClient('http://localhost:9753/',['echoInteger','echoSimpleClass','echoSimpleClassArray'])
	print c.echoInteger(make('echoInteger',i=3)).retval
	
	print c.echoSimpleClass(make('echoSimpleClass',sc=make('SimpleClass',i=34,s='bobo'))).retval.s
	
	d = c.echoSimpleClassArray(make('echoSimpleClassArray',
	                sca=[
                        make('SimpleClass',i=34,s='jasdf'),
                        make('SimpleClass',i=34,s='bobo'),
                        make('SimpleClass',i=34,s='poo'), 
	                    ]
	                ))
	print '*'*80
	for sc in d.retval:
	    print sc.s
	




from soaplib.xml import create_xml_element, create_xml_subelement
from soaplib.xml import ElementTree, NamespaceLookup
from soaplib.soap import Message, MethodDescriptor

from code import interact

def soapmethod(*params, **kparams):
    '''
    This is a method decorator to flag a method as a soap 'rpc' method.  It
    will behave like a normal python method on a class, and will only behave
    differently when the keyword '_soap_descriptor' is passed in, returning
    a 'MethodDescriptor' object.  This decorator does none of the soap/xml 
    serialization, only flags a method as a soap method.  This decorator
    should only be used on methods that are in instances of SoapServiceBase.
    '''
    def explain(f):
        def explainMethod(*args, **kwargs):
            if kwargs.has_key('_soap_descriptor'):
                name = f.func_name
            
                _returns = kparams.get('_returns')
                _isCallback = kparams.get('_isCallback',False)
                _soapAction = kparams.get('_soapAction',name)
                _isAsync = kparams.get('_isAsync',False)
                _inMessage = kparams.get('_inMessage',name)
                _inVariableNames = kparams.get('_inVariableNames',{})
                _outMessage = kparams.get('_outMessage','%sResponse'%name)
                _outVariableName = kparams.get('_outVariableName', '%sResult'%name)
                _mtom = kparams.get('_mtom',False)
                
                ns = None

                # the decorator function does not have a reference to the
                # class and needs to be passed in
                if kwargs.has_key('klazz'):
                    ns = getTNS(kwargs['klazz'])

                # input message
                param_names = f.func_code.co_varnames[1:f.func_code.co_argcount]
                try:
                    in_params = [(_inVariableNames.get(param_names[i], param_names[i]), params[i]) for i in range(0, len(params))]
                except IndexError,e:
                    print f.func_name
                    raise Exception("%s has parameter numbers mismatching" %f.func_name)

                in_message = Message(_inMessage,in_params,ns=ns,typ=_inMessage)
                
                # output message
                if _returns:
                    out_params = [(_outVariableName,_returns)]
                else:
                    out_params = []
                out_message = Message(_outMessage,out_params,ns=ns,typ=_outMessage)
                doc = getattr(f,'__doc__')
                descriptor = MethodDescriptor(f.func_name,_soapAction,in_message,out_message,doc,_isCallback,_isAsync,_mtom)
                
                return descriptor
            return f(*args, **kwargs)
        explainMethod.__doc__ = f.__doc__
        explainMethod.func_name = f.func_name
        explainMethod._is_soap_method = True
        return explainMethod
    return explain

def getTNS(cls):
    '''
    Utility function to get the namespace of a given service class
    @param the service in question
    @return the namespace
    '''
    serviceName = cls.__name__.split('.')[-1]
    if hasattr(cls,'__tns__'):
        return cls.__tns__
    if cls.__module__ == '__main__':
        return '.'.join((serviceName,serviceName))
    return '.'.join((cls.__module__,serviceName))

class SoapServiceBase(object):
    '''
    This class serves as the base for all soap services.  Subclasses of this 
    class will use the soapmethod and soapdocument decorators to flag methods
    to be exposed via soap.  This class is responsible for generating the 
    wsdl for this object.
    '''
    def __init__(self):
        self._soap_methods = []
        self.__wsdl__ = None
        self.__tns__ = getTNS(self.__class__)
        self._soap_methods = self._get_soap_methods()

    def _get_soap_methods(self):
        '''Returns a list of method descriptors for this object'''
        soap_methods = []
        for funcName in dir(self):
            func = getattr(self,funcName)
            if callable(func) and hasattr(func,'_is_soap_method'):
                descriptor = func(_soap_descriptor=True,klazz=self.__class__)
                soap_methods.append(descriptor)
        return soap_methods
            
    def methods(self):
        '''
        returns the soap methods for this object
        @return method descriptor list
        '''
        return self._soap_methods

    def _hasCallbacks(self):
        '''Determines if this object has callback methods or not'''
        for method in self.methods():
            if method.isCallback: 
                return True
        return False

    def header_objects(self):
        return []

    def getServiceNames(self):
        '''Returns the service name(s) for this service. If this
        object has callbacks, then a second service is declared in
        the wsdl for those callbacks'''
        serviceName = self.__class__.__name__.split('.')[-1]
        if self._hasCallbacks():
            return [serviceName,'%sCallback'%serviceName]
        return [serviceName]

    def wsdl(self, url):
        '''
        This method generates and caches the wsdl for this object based
        on the soap methods designated by the soapmethod or soapdocument
        descriptors
        @param url the url that this service can be found at.  This must be 
        passed in by the caller because this object has no notion of the
        server environment in which it runs.
        @returns the string of the wsdl
        '''
        if not self.__wsdl__ == None:
            # return the cached __wsdl__
            return self.__wsdl__
        url = url.replace('.wsdl','')
        # otherwise build it
        serviceName = self.__class__.__name__.split('.')[-1]

        tns = self.__tns__
        methods = self.methods()
        hasCallbacks = self._hasCallbacks() 
        
        nsmap = NamespaceLookup(tns, True)
        if hasCallbacks:
            nsmap.set('wsa','http://schemas.xmlsoap.org/ws/2003/03/addressing')
        
        root = create_xml_element(
            "definitions", nsmap, 'http://schemas.xmlsoap.org/wsdl/')
        root.set('targetNamespace',tns)
        root.set('name',serviceName)
        
        types = create_xml_subelement(root, "types")

        self._add_schema(types, methods, nsmap)
        self._add_messages_for_methods(root, methods, nsmap)

        # add necessary async headers
        # WS-Addressing -> RelatesTo ReplyTo MessageID
        # callback porttype
        if hasCallbacks:
            wsaSchemaNode = create_xml_subelement(types, "schema")
            wsaSchemaNode.set("targetNamespace", tns+'Callback')
            wsaSchemaNode.set("xmlns", "http://www.w3.org/2001/XMLSchema")

            importNode = create_xml_subelement(wsaSchemaNode, "import")
            importNode.set("namespace", "http://schemas.xmlsoap.org/ws/2003/03/addressing")
            importNode.set("schemaLocation", "http://schemas.xmlsoap.org/ws/2003/03/addressing/")

        
            reltMessage = create_xml_subelement(root,'message')
            reltMessage.set('name','RelatesToHeader')
            reltPart = create_xml_subelement(reltMessage,'part')
            reltPart.set('name','RelatesTo')
            reltPart.set('element','wsa:RelatesTo')

            replyMessage = create_xml_subelement(root,'message')
            replyMessage.set('name','ReplyToHeader')
            replyPart = create_xml_subelement(replyMessage,'part')
            replyPart.set('name','ReplyTo')
            replyPart.set('element','wsa:ReplyTo')

            idHeader = create_xml_subelement(root,'message')
            idHeader.set('name','MessageIDHeader')
            idPart = create_xml_subelement(idHeader,'part')
            idPart.set('name','MessageID')
            idPart.set('element','wsa:MessageID')

            # make portTypes
            callbackPortType = create_xml_subelement(root,'portType')
            callbackPortType.set('name','%sCallback'%serviceName)
            
            cbServiceName = '%sCallback'%serviceName
            cbService = create_xml_subelement(root,'service')
            cbService.set('name',cbServiceName)
            cbWsdlPort = create_xml_subelement(cbService,'port')
            cbWsdlPort.set('name',cbServiceName)
            cbWsdlPort.set('binding','tns:%s'%cbServiceName)
            cbAddr = create_xml_subelement(cbWsdlPort, nsmap.get('soap') + 'address')
            cbAddr.set('location',url)
            
            
        serviceName = self.__class__.__name__.split('.')[-1] 
        portType = create_xml_subelement(root,'portType')
        portType.set('name',serviceName)
        for method in methods:
            if method.isCallback:
                operation = create_xml_subelement(callbackPortType,'operation')
            else:
                operation = create_xml_subelement(portType,'operation')
                
            operation.set('name',method.name)
            params = []
            for name,param in method.inMessage.params:
                params.append(name)

            documentation = create_xml_subelement(operation,'documentation')
            documentation.text = method.doc
            operation.set('parameterOrder',method.inMessage.typ)
            opInput = create_xml_subelement(operation,'input')
            opInput.set('name',method.inMessage.typ)
            opInput.set('message','tns:%s'%method.inMessage.typ)

            if method.outMessage.params != None and not method.isCallback and not method.isAsync:
                opOutput = create_xml_subelement(operation,'output')
                opOutput.set('name',method.outMessage.typ)
                opOutput.set('message','tns:%s'%method.outMessage.typ)
        
        # make partner link
        plink = create_xml_subelement(root, nsmap.get('plnk') + 'partnerLinkType')
        plink.set('name',serviceName)
        role = create_xml_subelement(plink, nsmap.get('plnk') + 'role')
        role.set('name', serviceName)
        plinkPortType = create_xml_subelement(role, nsmap.get('plnk') + 'portType')
        plinkPortType.set('name','tns:%s'%serviceName) 

        if hasCallbacks:
            role = create_xml_subelement(plink, nsmap.get('plnk') + 'role')
            role.set('name', '%sCallback'%serviceName)
            plinkPortType = create_xml_subelement(role, nsmap.get('plnk') + 'portType')
            plinkPortType.set('name','tns:%sCallback'%serviceName) 

        self._add_bindings_for_methods(root, serviceName, methods, nsmap)

        service = create_xml_subelement(root,'service')
        service.set('name',serviceName)
        wsdlPort = create_xml_subelement(service,'port')
        wsdlPort.set('name',serviceName)
        wsdlPort.set('binding','tns:%s'%serviceName)
        addr = create_xml_subelement(wsdlPort, nsmap.get('soap') + 'address')
        addr.set('location',url)

        wsdl = ElementTree.tostring(root)
        wsdl = "<?xml version='1.0' encoding='utf-8' ?>%s"%(wsdl)

        #cache the wsdl for next time
        self.__wsdl__ = wsdl 
        return self.__wsdl__

    def _add_schema(self, types, methods, nsmap):
        '''A private method for adding the appropriate entries
        to the schema for the types in the specified methods
        @param the schema node to add the schema elements to
        @param the list of methods.
        '''
        schema_entries = {}
        for method in methods:
            params = method.inMessage.params 
            returns = method.outMessage.params

            for name,param in params:
                param.add_to_schema(schema_entries, nsmap)

            if returns:
                returns[0][1].add_to_schema(schema_entries, nsmap)

            method.inMessage.add_to_schema(schema_entries, nsmap)
            method.outMessage.add_to_schema(schema_entries, nsmap)


        schemaNode = create_xml_subelement(types, "schema")
        schemaNode.set("targetNamespace", self.__tns__)
        schemaNode.set("xmlns", "http://www.w3.org/2001/XMLSchema")
        
        for xxx, node in schema_entries.items():
            schemaNode.append(node)
                        
        return schemaNode

    def _add_messages_for_methods(self, root, methods, nsmap):
        '''
        A private method for adding message elements to the wsdl
        @param the the root element of the wsdl
        @param the list of methods.        
        '''
        messages = []
        #make messages
        for method in methods:
            methodName = method.name
            # making in part
            inMessage  = create_xml_element('message', nsmap)
            inMessage.set('name',method.inMessage.typ)

            if len(method.inMessage.params) > 0:
                inPart = create_xml_subelement(inMessage,'part')
                inPart.set('name',method.inMessage.name)
                inPart.set('element', 'tns:' + method.inMessage.typ)

            messages.append(inMessage)

            # making out part                
            outMessage = create_xml_element('message', nsmap)
            outMessage.set('name',method.outMessage.typ)
            if len(method.outMessage.params) > 0:
                outPart = create_xml_subelement(outMessage,'part')
                outPart.set('name', method.outMessage.name)
                outPart.set('element', 'tns:' + method.outMessage.typ)
            messages.append(outMessage)
            
        for message in messages:
            root.append(message)


    def _add_bindings_for_methods(self, root, serviceName, methods, nsmap):
        '''
        A private method for adding bindings to the wsdld
        @param the root element of the wsdl
        @param the name of this service
        @param the methods to be add to the binding node
        '''
        hasCallbacks = self._hasCallbacks()
    
        # make binding
        binding = create_xml_subelement(root,'binding')
        binding.set('name',serviceName)
        binding.set('type','tns:%s'%serviceName)
        
        sbinding = create_xml_subelement(binding, nsmap.get('soap') + 'binding')
        sbinding.set('style','document')
        sbinding.set('transport','http://schemas.xmlsoap.org/soap/http')

        if hasCallbacks:
            callbackBinding = create_xml_subelement(root,'binding')
            callbackBinding.set('name','%sCallback'%serviceName)
            callbackBinding.set('type','typens:%sCallback'%serviceName)

            sbinding = create_xml_subelement(callbackBinding, nsmap.get('soap') + 'binding')
            sbinding.set('transport','http://schemas.xmlsoap.org/soap/http')

        for method in methods:
            operation = create_xml_element('operation', nsmap)
            operation.set('name',method.name)

            soapOperation = create_xml_subelement(operation, nsmap.get('soap') + 'operation')
            soapOperation.set('soapAction',method.soapAction)

            soapOperation.set('style','document')

            input = create_xml_subelement(operation,'input')
            input.set('name',method.inMessage.typ)
            soapBody = create_xml_subelement(input, nsmap.get('soap') + 'body')
            soapBody.set('use','literal')

            if method.outMessage.params != None and not method.isAsync and not method.isCallback:
                output = create_xml_subelement(operation,'output')
                output.set('name',method.outMessage.typ)
                soapBody = create_xml_subelement(output, nsmap.get('soap') + 'body')
                soapBody.set('use','literal')

            if method.isCallback:
                relatesTo = create_xml_subelement(input, nsmap.get('soap') + 'header')
                relatesTo.set('message','tns:RelatesToHeader')
                relatesTo.set('part','RelatesTo')
                relatesTo.set('use','literal')

                callbackBinding.append(operation)
            else:
                if method.isAsync:
                    rtHeader = create_xml_subelement(input, nsmap.get('soap') + 'header')
                    rtHeader.set('message','tns:ReplyToHeader')
                    rtHeader.set('part','ReplyTo')
                    rtHeader.set('use','literal')

                    midHeader = create_xml_subelement(input, nsmap.get('soap') + 'header')
                    midHeader.set('message','tns:MessageIDHeader')
                    midHeader.set('part','MessageID')
                    midHeader.set('use','literal')

                binding.append(operation)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

class _Missing(object):

    def __repr__(self):
        return 'no value'

    def __reduce__(self):
        return '_missing'

_missing = _Missing()

class cached_property(object):
    """A decorator that converts a function into a lazy property.  The
    function wrapped is called the first time to retrieve the result
    and then that calculated result is used the next time you access
    the value::

        class Foo(object):

            @cached_property
            def foo(self):
                # calculate something important here
                return 42

    The class has to have a `__dict__` in order for this property to
    work.

    .. versionchanged:: 0.6
       the `writeable` attribute and parameter was deprecated.  If a
       cached property is writeable or not has to be documented now.
       For performance reasons the implementation does not honor the
       writeable setting and will always make the property writeable.
    """

    # implementation detail: this property is implemented as non-data
    # descriptor.  non-data descriptors are only invoked if there is
    # no entry with the same name in the instance's __dict__.
    # this allows us to completely get rid of the access function call
    # overhead.  If one choses to invoke __get__ by hand the property
    # will still work as expected because the lookup logic is replicated
    # in __get__ for manual invocation.

    def __init__(self, func, name=None, doc=None, writeable=False):
        if writeable:
            from warnings import warn
            warn(DeprecationWarning('the writeable argument to the '
                                    'cached property is a noop since 0.6 '
                                    'because the property is writeable '
                                    'by default for performance reasons'))

        self.__name__ = name or func.__name__
        self.__module__ = func.__module__
        self.__doc__ = doc or func.__doc__
        self.func = func

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        value = obj.__dict__.get(self.__name__, _missing)
        if value is _missing:
            value = self.func(obj)
            obj.__dict__[self.__name__] = value
        return value




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# created: zhangpeng <zhangpeng@ivtime.com>

from torweb import script
from code import interact


class Command(object):
    def __init__(self, app):
        from torweb import make_application
        self.app = app
        self.application = application = make_application(app)
        self._actions = {
                'urls' :   urls(application),
        }
        for name, action in getattr(app, "actions", []):
            self._actions[name] = action(application)

    def run(self):
        script.run(self._actions, '')

    def register(self, func, name=None):
        name = name if name is not None else func.func_name
        self._actions[name] = func

def urls(app):
    def action():
        """show all urls"""
        for spec in app.url_handlers:
            print "%s\t\t\t%s(%s)"%(spec.regex.pattern, spec.handler_class, spec.kwargs)

    return action




import base64
import cStringIO
from soaplib.xml import ns, create_xml_element

class Attachment(object):

    def __init__(self,data=None,fileName=None):
        self.data = data
        self.fileName = fileName

    def save_to_file(self):
        '''This method writes the data to the specified file.  This method
        assumes that the filename is the full path to the file to be written.
        This method also assumes that self.data is the base64 decoded data,
        and will do no additional transformations on it, simply write it to
        disk.
        '''
        if not self.data:
            raise Exception("No data to write")
        if not self.fileName:
            raise Exception("No filename specified")
        f = open(self.fileName,'wb')
        f.write(self.data)
        f.flush()
        f.close()

    def load_from_file(self):
        '''
        This method loads the data from the specified file, and does
        no encoding/decoding of the data
        '''
        if not self.fileName:
            raise Exception("No filename specified")
        f = open(self.fileName,'rb')
        self.data = f.read()
        f.close()

    @classmethod
    def to_xml(cls,value,name='retval',nsmap=ns):
        '''This class method takes the data from the attachment and 
        base64 encodes it as the text of an Element.  An attachment can
        specify a filename and if no data is given, it will read the data 
        from the file
        '''
        if value.__class__ is not Attachment:
            raise Exception("Do not know how to serialize class %s"%type(value))
            
        element = create_xml_element(name, nsmap)
        if value.data:
            # the data has already been loaded, just encode
            # and return the element
            element.text = base64.encodestring(value.data)
        elif value.fileName:
            # the data hasn't been loaded, but a file has been
            # specified
            data_string = cStringIO.StringIO()

            fileName = value.fileName
            file = open(fileName,'rb')
            base64.encode(file, data_string)
            file.close()

            # go back to the begining of the data
            data_string.seek(0)
            element.text = str(data_string.read())
        else:
            raise Exception("Neither data nor a filename has been specified")
        
        return element    
    
    @classmethod
    def from_xml(cls,element):
        '''
        This method returns an Attachment object that contains
        the base64 decoded string of the text of the given element
        '''
        data = base64.decodestring(element.text)
        a = Attachment(data=data)
        return a

    @classmethod
    def get_datatype(cls,nsmap=None):
        '''Returns the datatype base64Binary'''
        if nsmap is not None:
            return nsmap.get(cls.get_namespace_id()) + 'base64Binary'
        return 'base64Binary'
        
    @classmethod
    def get_namespace_id(cls):
        return 'xs'

    @classmethod
    def add_to_schema(cls,added_params,nsmap):
        ''' 
        Nothing needs to happen here as base64Binary is a standard
        schema element
        '''
        pass

    @classmethod
    def collect_namespaces(cls, ns_map):
        pass

if __name__ == '__main__':
    import os
    os.system('rm /tmp/boxer2.tiff')

    a = Attachment(fileName='/tmp/boxer.tiff')
    a.load_data()
    #print a.data
    element = Attachment.to_xml(a,'bob')
    a1 = Attachment.from_xml(element)

    a1.fileName = '/tmp/boxer2.tiff'
    a1.save_data()

    os.system('open /tmp/boxer2.tiff')

    







#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import sys
import urllib
import soaplib
import urllib

from lxml import etree

def create_relates_to_header(relatesTo, attrs={}):
    '''Creates a 'relatesTo' header for async callbacks'''
    relatesToElement = etree.Element(
        '{%s}RelatesTo' % soaplib.ns_wsa)
    for k, v in attrs.items():
        relatesToElement.set(k, v)
    relatesToElement.text = relatesTo
    return relatesToElement


def create_callback_info_headers(message_id, reply_to):
    '''Creates MessageId and ReplyTo headers for initiating an
    async function'''
    message_id = etree.Element('{%s}MessageID' % soaplib.ns_wsa)
    message_id.text = message_id

    reply_to = etree.Element('{%s}ReplyTo' % soaplib.ns_wsa)
    address = etree.SubElement(reply_to, '{%s}Address' % soaplib.ns_wsa)
    address.text = reply_to

    return message_id, reply_to

def get_callback_info(request):
    '''
    Retrieves the messageId and replyToAddress from the message header.
    This is used for async calls.
    '''
    message_id = None
    reply_to_address = None
    header = request.soap_req_header

    if header:
        headers = header.getchildren()
        for header in headers:
            if header.tag.lower().endswith("messageid"):
                message_id = header.text

            if header.tag.lower().find("replyto") != -1:
                replyToElems = header.getchildren()

                for replyTo in replyToElems:
                    if replyTo.tag.lower().endswith("address"):
                        reply_to_address = replyTo.text

    return message_id, reply_to_address

def get_relates_to_info(request):
    '''Retrieves the relatesTo header. This is used for callbacks'''
    header = request.soap_req_header
    if header:
        headers = header.getchildren()
        for header in headers:
            if header.tag.lower().find('relatesto') != -1:
                return header.text

def split_url(url):
    '''Splits a url into (uri_scheme, host[:port], path)'''
    scheme, remainder = urllib.splittype(url)
    host, path = urllib.splithost(remainder)
    return scheme.lower(), host, path

def reconstruct_url(environ):
    '''
    Rebuilds the calling url from values found in the
    environment.

    This algorithm was found via PEP 333, the wsgi spec and
    contributed by Ian Bicking.
    '''

    url = environ['wsgi.url_scheme'] + '://'

    if environ.get('HTTP_HOST'):
        url += environ['HTTP_HOST']

    else:
        url += environ['SERVER_NAME']

        if environ['wsgi.url_scheme'] == 'https':
            if environ['SERVER_PORT'] != '443':
                url += ':' + environ['SERVER_PORT']

        else:
            if environ['SERVER_PORT'] != '80':
                url += ':' + environ['SERVER_PORT']

    if (urllib.quote(environ.get('SCRIPT_NAME', '')) == '/' and
        urllib.quote(environ.get('PATH_INFO', ''))[0:1] == '/'):
        #skip this if it is only a slash
        pass

    elif urllib.quote(environ.get('SCRIPT_NAME', ''))[0:2] == '//':
        url += urllib.quote(environ.get('SCRIPT_NAME', ''))[1:]

    else:
        url += urllib.quote(environ.get('SCRIPT_NAME', ''))

    url += urllib.quote(environ.get('PATH_INFO', ''))
    if environ.get('QUERY_STRING'):
        url += '?' + environ['QUERY_STRING']

    return url

def check_pyversion(*minversion):
    return sys.version_info[:3] >= minversion




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

class duration(object):
    __seq1 = [("Y", "years"), ("M", "months"), ("D", "days")]
    __seq2 = [("H", "hours"), ("M", "minutes"), ("S", "seconds")]

    def __init__(self, years = 0, months = 0, days = 0, hours = 0, minutes = 0, seconds = 0, negative = False):
        self.years = years
        self.months = months
        self.days = days
        self.hours = hours
        self.minutes = minutes
        self.seconds = seconds
        self.negative = negative

    def __str__(self):
        def tostr(seq):
            str = ""
            for s, attr in seq:
                n = getattr(self, attr)
                assert n >= 0 and (n == round(n) or attr == "seconds")
                if n:
                    str += (n == round(n) and "%i%s" or "%s%s") % (n, s)
            return str

        s1 = tostr(self.__seq1)
        s2 = tostr(self.__seq2)
        if s2:
            ret = "P%sT%s" % (s1, s2)
        elif s1:
            ret = "P%s" % s1
        else:
            ret = "PT0S"
        if self.negative:
            ret = "-" + ret
        return ret

    def __repr__(self):
        return "%s(%s, %s, %s, %s, %s, %s, %s)" % (self.__class__.__name__,
                self.years, self.months, self.days,
                self.hours, self.minutes, self.seconds,
                self.negative)

    @classmethod
    def parse(cls, str):
        orig_str = str
        ret = cls()

        def parse_token(str):
            n = "0"
            for i in range(len(str)):
                if str[i].isdigit() or str[i] == ".":
                    n += str[i]
                else:
                    break
            return (float(n), str[i], str[i+1:])

        def parse_seq(str, seq):
            i = 0
            while str:
                n, s, str = parse_token(str)
                while i < len(seq):
                    s1, attr = seq[i]
                    i += 1
                    if s == s1:
                        if n >= 0 and (n == round(n) or attr == "seconds"):
                            setattr(ret, attr, n)
                            break
                        else:
                            raise Exception
                else:
                    raise Exception

        try:
            if str.startswith("-"):
                ret.negative = True
                str = str[1:]
            if str.startswith("P"):
                str = str[1:]
            else:
                raise Exception
            str1, _, str2 = str.partition("T")
            if not (str1 or str2):
                raise Exception
            parse_seq(str1, cls.__seq1)
            parse_seq(str2, cls.__seq2)
        except:
            raise ValueError("Duration [%s] not in correct format" % orig_str)

        return ret




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""
The soaplib.serializers.table module is EXPERIMENTAL. It does not support
inheritance, it is supposedly buggy and possibly slow.
"""

import logging
logger = logging.getLogger(__name__)

import sqlalchemy
from sqlalchemy import Column

from sqlalchemy.ext.declarative import DeclarativeMeta
from soaplib.serializers.clazz import TypeInfo
from soaplib.serializers.clazz import ClassSerializerBase
from soaplib.serializers import primitive
from soaplib.serializers import clazz

_type_map = {
    sqlalchemy.Text: primitive.String,
    sqlalchemy.String: primitive.String,
    sqlalchemy.Unicode: primitive.String,
    sqlalchemy.UnicodeText: primitive.String,

    sqlalchemy.Float: primitive.Float,
    sqlalchemy.Numeric: primitive.Double,
    sqlalchemy.Integer: primitive.Integer,
    sqlalchemy.SmallInteger: primitive.Integer,

    sqlalchemy.Boolean: primitive.Boolean,
    sqlalchemy.DateTime: primitive.DateTime,
    sqlalchemy.orm.relation: clazz.Array,
}

def parse_cls_dict(cls_dict):
    cls_dict["_type_info"] = _type_info = TypeInfo()

    for k, v in cls_dict.items():
        if not k.startswith('__'):
            if isinstance(v, Column):
                if v.type in _type_map:
                    rpc_type = _type_map[v.type]
                elif type(v.type) in _type_map:
                    rpc_type = _type_map[type(v.type)]
                else:
                    raise Exception("soap_type was not found. maybe "
                                    "_type_map needs a new entry.")

                _type_info[k]=rpc_type

class TableSerializerMeta(DeclarativeMeta):
    def __new__(cls, cls_name, cls_bases, cls_dict):
        if cls_dict.get("__type_name__", None) is None:
            cls_dict["__type_name__"] = cls_name

        if cls_dict.get("_type_info", None) is None:
            parse_cls_dict(cls_dict)

        return DeclarativeMeta.__new__(cls, cls_name, cls_bases, cls_dict)

class TableSerializer(ClassSerializerBase):
    __metaclass__ = TableSerializerMeta
    _decl_class_registry={}
